my_heatmap <- function (x,
                        breaks, 
                        col = "heat.colors", 
                        na.rm = TRUE, 
                        rowsep, 
                        sepcolor = "white", 
                        sepwidth = c(0.05, 0.05), 
                        cexRow = 0.2 + 1/log10(nr),
                        cexCol = 0.2 + 1/log10(nc),
                        na.color = "gray", 
                        hline = median(breaks), 
                        vline = median(breaks), 
                        linecol = "black",
                        margins = c(5, 5), 
                        labRow = NULL, 
                        labCol = NULL, 
                        key = TRUE,
                        keysize = 1.5, 
                        symkey = min(x < 0, na.rm = TRUE), 
                        main = NULL,
                        xlab = NULL,
                        ylab = NULL, 
                        lmat = NULL,
                        lhei = NULL,
                        lwid = NULL,
                        ...) 
{
  scale01 <- function(x, low = min(x), high = max(x)) {
    x <- (x - low)/(high - low)
    x
  }
  if (!missing(breaks)) 
    warning("Using scale=\"row\" or scale=\"column\" when breaks are", 
            "specified can produce unpredictable results.", "Please consider using only one or the other.")
  if (length(di <- dim(x)) != 2 || !is.numeric(x)) 
    stop("`x' must be a numeric matrix")
  nr <- di[1]
  nc <- di[2]
  if (nr <= 1 || nc <= 1) 
    stop("`x' must have at least 2 rows and 2 columns")
  if (!is.numeric(margins) || length(margins) != 2) 
    stop("`margins' must be a numeric vector of length 2")
  rowInd <- nr:1
  colInd <- 1:nc
  x <- x[rowInd, colInd]
  x.unscaled <- x
  if (is.null(labRow)) 
    labRow <- if (is.null(rownames(x))) 
      (1:nr)[rowInd]
    else rownames(x)
  else labRow <- labRow[rowInd]
  if (is.null(labCol)) 
    labCol <- if (is.null(colnames(x))) 
      (1:nc)[colInd]
    else colnames(x)
  else labCol <- labCol[colInd]
  if (missing(breaks) || is.null(breaks) || length(breaks) < 1) 
    if (missing(col)) 
      breaks <- 16
    else breaks <- length(col) + 1
  if (length(breaks) == 1) {
    breaks <- seq(0, max(x, na.rm = na.rm), length = breaks)
  }
  nbr <- length(breaks)
  ncol <- length(breaks) - 1
  if (class(col) == "function") 
    col <- col(ncol)
  else if (is.character(col) && length(col) == 1) 
    col <- do.call(col, list(ncol))
  min.breaks <- min(breaks)
  max.breaks <- max(breaks)
  x[] <- ifelse(x < min.breaks, min.breaks, x)
  x[] <- ifelse(x > max.breaks, max.breaks, x)
  if (missing(lhei) || is.null(lhei)) 
    lhei <- c(keysize, 4)
  if (missing(lwid) || is.null(lwid)) 
    lwid <- c(keysize, 4)
  if (missing(lmat) || is.null(lmat)) {
    lmat <- rbind(4:3, 2:1)
    lmat[is.na(lmat)] <- 0
  }
  if (length(lhei) != nrow(lmat)) 
    stop("lhei must have length = nrow(lmat) = ", nrow(lmat))
  if (length(lwid) != ncol(lmat)) 
    stop("lwid must have length = ncol(lmat) =", ncol(lmat))
  op <- par(no.readonly = TRUE)
  on.exit(par(op))
  layout(matrix(data=c(1,2), nrow=1, ncol=2), widths=c(5,1), heights=c(1))
  par(mar = c(5, 4, 7, 2))
  iy <- 1:nr
  image(1:nc, 1:nr, x, xlim = 0.5 + c(0, nc), ylim = 0.5 + 
        c(0, nr), axes = FALSE, xlab = "", ylab = "", col = col, 
        breaks = breaks, ...)
  if (!invalid(na.color) & any(is.na(x))) {
    mmat <- ifelse(is.na(x), 1, NA)
    image(1:nc, 1:nr, mmat, axes = FALSE, xlab = "", ylab = "", 
          col = na.color, add = TRUE)
  }
  axis(1, 1:nc, labels = labCol, las = 2, line = -0.5, tick = 0, 
       cex.axis = cexCol)
  if (!is.null(xlab)) 
    mtext(xlab, side = 1, line = margins[1] - 1.25)
  axis(4, 1:nr, labels = labRow, las = 2, line = -0.5, tick = 0, 
       cex.axis = cexRow)
  if (!is.null(ylab)) 
    mtext(ylab, side = 4, line = margins[2] - 1.25)
##   if (!missing(add.expr)) 
##     eval(substitute(add.expr))
  if (!missing(rowsep)) 
    for (rsep in rowsep)
      rect(xleft = 0,
           ybottom = (ncol(x) + 1 - rsep) - 0.5,
           xright = nrow(x) + 1,
           ytop = (ncol(x) + 1 - rsep) - 0.5 - sepwidth[2],
           lty = 1, lwd = 1, col = sepcolor, border = sepcolor)
  min.scale <- min(breaks)
  max.scale <- max(breaks)
  x.scaled <- scale01(t(x), min.scale, max.scale)
  if (!is.null(vline)) {
    vline.vals <- scale01(vline, min.scale, max.scale)
    abline( v = vline.vals, col = linecol, lty = 1 )
  }
  for (i in colInd) {
    if (!is.null(vline)) {
      vline.vals <- scale01(vline, min.scale, max.scale)
      abline(v = i + vline.vals, col = linecol, lty = 1)
    }
    xv <- rep(i, nrow(x.scaled)) + x.scaled[, i] - 0.5
    xv <- c(xv[1], xv)
    yv <- 1:length(xv) - 0.5
  }
  if (!is.null(hline)) {
    hline.vals <- scale01(hline, min.scale, max.scale)
    abline(h = hline.vals, col = linecol, lty = 1)
  }
  for (i in rowInd) {
    if (!is.null(hline)) {
      hline.vals <- scale01(hline, min.scale, max.scale)
      abline(h = i + hline.vals, col = linecol, lty = 1)
    }
    yv <- rep(i, ncol(x.scaled)) + x.scaled[i, ] - 0.5
    yv <- rev(c(yv[1], yv))
    xv <- length(yv):1 - 0.5
  }
  if (!is.null(main)) 
    title(main, cex.main = 1.5 * op[["cex.main"]])
  if (key) {
    par(mar = c(5, 4, 7, 2))
    if (symkey) {
      max.raw <- max(abs(x), na.rm = TRUE)
      min.raw <- -max.raw
    }
    else {
      min.raw <- min(x, na.rm = TRUE)
      max.raw <- max(x, na.rm = TRUE)
    }
    z <- seq(0, max.raw, length = length(col))
    par(mar = c(3,2.5,2.5,2))
    image(1, z, matrix(data=z, ncol=length(z), nrow=1), 
          xlab="", ylab="", xaxt = "n", col = col, breaks = breaks)
  }
  else plot.new()
  invisible(list(rowInd = rowInd, colInd = colInd))
}
