#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_n);
&getopts('hi:j:n:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a file contaning a list of scalar values
# -j   a file contaning a list of scalar values
# -n   jobname used for output

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $file_1  = $opt_i or die $usage;
my $file_2  = $opt_j or die $usage;
my $jobname = $opt_n or die $usage;
my $datfile = $jobname . '.dat';
my $gnufile = $jobname . '.gnu';
my $epsfile = $jobname . '.eps';

# Load input files
my @vals_1  = read_value_file( $file_1 );
my $num_1   = @vals_1;
print STDERR $num_1 . ' values were foune in ' . $file_1 . "\n";
my @vals_2  = read_value_file( $file_2 );
my $num_2   = @vals_2;
print STDERR $num_2 . ' values were found in ' . $file_2 . "\n";

# Draw 2d plot
print_data( $datfile, \@vals_1, \@vals_2 );
my $num_dat = wc( $datfile );
print STDERR $num_dat . " data points were printed : " . $datfile . "\n";
print_gnu_file( $datfile, $gnufile, $epsfile );
`gnuplot $gnufile`;


sub read_value_file {
    my $file = shift;
    my @vals;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split("\t", $_);
	if( @cols == 1 ){ push( @vals, $cols[0] ); }
    }
    close IN;
    return @vals;
}


sub print_data {
    my $file   = shift;
    my $vals_1 = shift;
    my $vals_2 = shift;
    open(DAT, ">$file") or die "Can't open $file to write.";
    if( @$vals_1 != @$vals_2 ){
	die "Conflicting number of values:";
    }
    for( my $i=0 ; $i<@$vals_1 ; ++$i ){
	my $val1 = $$vals_1[$i];
	my $val2 = $$vals_2[$i];
	unless( ($val1 eq 'N/A') or ($val2 eq 'N/A') ){
	    print DAT $val1 . "\t" . $val2 . "\n";
	}
    }
    close DAT;
}


sub wc {
    my $file = shift;
    my $wc   = `wc -l $file`;
    my @wc   = split(" ", $wc);
    return $wc[0];
}


sub print_gnu_file {
    my $datfile = shift;
    my $gnufile = shift;
    my $epsfile = shift;
    open(GNU, ">$gnufile") or die "Can't open $gnufile to write.";
    print GNU 'set terminal postscript color'                         . "\n"
	. 'set output "' . $epsfile . '"'                             . "\n"
	. 'plot "' . $datfile . '" with points'
	. ' pointtype 1 pointsize 1.0'                                . "\n";
    close GNU;
}
