#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_d);
&getopts('hi:j:d:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   an organism name (e.g. hsa)
# -j   an organism name (e.g. mmu)
# -d   a directory containing pamdist files

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
my $scr_dir = '/Users/hacchy/Documents/slab-works/oasys/oasys_v0_2/scripts/';
my $mean_sd = $scr_dir . 'mean_sd.pl';
my $wilcox  = $scr_dir . 'wilcox.pl';
my $twoHist = $scr_dir . 'two-hists.pl';
die $usage if $opt_h;
my $org1    = $opt_i or die $usage;
my $org2    = $opt_j or die $usage;
my $pam_dir = $opt_d or die $usage;
$pam_dir   .= '/' unless $pam_dir =~ /\/$/;
my $ALLfile = $pam_dir . $org1 . '.vs.' . $org2 . '.all.dist';
my $IN_file = $pam_dir . $org1 . '.vs.' . $org2 . '.in.dist';
my $OUTfile = $pam_dir . $org1 . '.vs.' . $org2 . '.out.dist';
my $jobname = $pam_dir . $org1 . '.vs.' . $org2;

# Load input files
my $allfile = "/tmp/$org1.vs.$org2.$$.all.dist";
my $in_file = "/tmp/$org1.vs.$org2.$$.in.dist";
my $outfile = "/tmp/$org1.vs.$org2.$$.out.dist";
skip_na( $ALLfile, $allfile );
skip_na( $IN_file, $in_file );
skip_na( $OUTfile, $outfile );

# Compute and show statistics
my $num_all = wc( $allfile );
my $num_in  = wc( $in_file );
my $num_out = wc( $outfile );

print "\n"
    . '*** Number ***' . "\n"
    . 'All: ' . $num_all . "\n"
    . 'In : ' . $num_in  . "\t" . '(' . $num_in/$num_all*100  . '%)' . "\n"
    . 'Out: ' . $num_out . "\t" . '(' . $num_out/$num_all*100 . '%)' . "\n";

print "\n"
    . '*** Distance ***' . "\n"
    . 'All: ' . `$mean_sd -i $allfile`
    . 'In : ' . `$mean_sd -i $in_file`
    . 'Out: ' . `$mean_sd -i $outfile`;

print "\n"
    . '*** Statistical Test ***' . "\n"
    . 'Wilcox Test: ' . `$wilcox -i $in_file -j $outfile -m 2`;

print "\n"
    . '*** Draw Histogram ***' . "\n";
`$twoHist -i $in_file -j $outfile -n 10 -o $jobname`;

print "\n"
    . '*** Remove temporal files ***' . "\n";
`rm $allfile`;
`rm $in_file`;
`rm $outfile`;


sub wc {
    my $file = shift;
    my $wc   = `wc -l $file`;
    my @wc   = split(" ", $wc);
    return $wc[0];
}


sub skip_na {
    my $in_file = shift;
    my $outfile = shift;
    open(IN, $in_file) or die "Can't open $in_file to read";
    open(OUT, ">$outfile") or die "Can't open $outfile to write.";
    print STDERR $in_file . "\t->\t" . $outfile . "\n";
    while(<IN>){
	chomp;
	my @cols = split(" ", $_);
	if( @cols == 1 ){
	    my $val = $cols[0];
	    print OUT $val . "\n" if( $val ne 'N/A' );
	}
    }
    close IN;
    close OUT;
}
