#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j);
&getopts('hi:j:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a file containing program results
# -j   a file containing answers

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $resFile = $opt_i or die $usage;
my $ansFile = $opt_j or die $usage;

# Read input files
my %resPair = read_pair_file( $resFile );
my %ansPair = read_pair_file( $ansFile );
my $num_res = keys %resPair;
my $num_ans = keys %ansPair;
print STDERR $num_res . ' pairs in result file' . "\n";
print STDERR $num_ans . ' pairs in answer file' . "\n";

# Print results
print_performance( \%resPair, \%ansPair );


sub read_pair_file {
    my $file = shift;
    my %pairs;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split("\t", $_);
	if(@cols>=2){
	    my ($g1, $g2, @trash) = @cols;
	    $pairs{$g1."\t".$g2}   = 1;
	}
    }
    close IN;
    return %pairs;
}


sub print_performance {
    my $resPair = shift;
    my $ansPair = shift;
    my ($tp, $fn) = (0, 0);
    foreach my $pair (sort keys %$ansPair){
	if( $$resPair{$pair} ){ ++$tp; }
	else                  { ++$fn; }
    }
    my $num_res = keys %$resPair;
    my $fp      = $num_res - $tp;
    my $sens    = $tp / ($tp+$fn);
    my $spec    = $tp / ($tp+$fp);
    print STDERR ""
	. "Sensitivity = " . $sens . " ($tp/" . ($tp+$fn) . ")" . "\n"
	. "Specificity = " . $spec . " ($tp/" . ($tp+$fp) . ")" . "\n";
}
