#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_o);
&getopts('hi:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a file containing orthologous gruops (*.groups)
# -o   an output file

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
my $gr_file = $opt_i or die $usage;
my $outFile = $opt_o or die $usage;

# Read input files
my %pairs   = read_groups_file( $gr_file );

# Output results
print_pairs( \%pairs, $outFile );


sub read_groups_file {
    my $file = shift;
    my %pairs;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split("\t", $_);
	if( @cols == 3 ){
	    my ($gr_id, $str1, $str2) = @cols;
	    my @g1_list = split(",", $str1);
	    my @g2_list = split(",", $str2);
	    foreach my $g1 (@g1_list){
		foreach my $g2 (@g2_list){
		    $pairs{$g1."\t".$g2} = 1;
		}
	    }
	}
    }
    close IN;
    return %pairs;
}


sub print_pairs {
    my $pairs = shift;
    my $file  = shift;
    my $dummy = 0.0;
    open(OUT, ">$file") or die "Can't open $file to write.";
    foreach my $pair (sort keys %$pairs){
	print OUT $pair . "\t" . $dummy . "\n";
    }
    close OUT;
}
