#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_o);
&getopts('hi:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i    yn00 result file
# -o    summary of Ka/Ks values

## Others:
# -h print help
_EOH_
    ;

die $usage if $opt_h;

my $infile=$opt_i or die $usage;
open(IN,$infile) or die "Can't open $infile to read.";
my $outfile=$opt_o or die $usage;
open(OUT,">$outfile") or die "Can't open $outfile to write.";

my ($NG86,$NG86_dN,$NG86_dS);
my ($YN00,$YN00_dN,$YN00_dS);
my ($LWL85,$LWL85_dN,$LWL85_dS);
my ($LWL85m,$LWL85m_dN,$LWL85m_dS);
my ($LPB93,$LPB93_dN,$LPB93_dS);
while(<IN>){
    chomp;
    if(/Nei & Gojobori 1986. dN\/dS \(dN, dS\)/){
	my $line=<IN>; 	$line=<IN>; $line=<IN>;
	if($line=~/^\S+ ([\d\.]+)\(([\d\.]+) ([\d\.]+)\)/){
	    ($NG86,$NG86_dN,$NG86_dS)=($1,$2,$3);
	} else {
	    ($NG86,$NG86_dN,$NG86_dS)=('-','-','-');
	}
    }
    if(/Yang & Nielsen \(2000\) method/){
	my $line=<IN>; $line=<IN>; $line=<IN>; $line=<IN>;
	$line=<IN>; $line=<IN>; $line=<IN>; $line=<IN>;
	my @cols=split(" ",$line);
	$YN00_dN=$cols[7];
	$YN00_dS=$cols[10];
	if(!defined($YN00_dN) or !defined($YN00_dS)){
	    ($YN00,$YN00_dN,$YN00_dS)=('-','-','-');
	    next;
	}

	if($YN00_dS!=0){
	    $YN00=round($YN00_dN/$YN00_dS,4);
	} else {
#	    print "YN00_dS=".$YN00_dS,"\n";
	    $YN00=0 if $YN00_dN==0;
	    $YN00='Inf' if $YN00_dN!=0;
	}
    }
    if(/^LWL85:/){
	my $line=$_;
	if($line=~/dS =\s+([\d\.]+) dN =\s+([\d\.]+)/){
	    ($LWL85_dN,$LWL85_dS)=($2,$1);
	} else {
	    ($LWL85,$LWL85_dN,$LWL85_dS)=('-','-','-');
	    next;
	}

	if($LWL85_dS!=0){
	    $LWL85=round($LWL85_dN/$LWL85_dS,4);
	} else {
	    $LWL85='-';
	}
    }
    if(/^LWL85m:/){
	my $line=$_;
	if($line=~/dS =\s+([\d\.]+) dN =\s+([\d\.]+)/){
	    ($LWL85m_dN,$LWL85m_dS)=($2,$1);
	} else {
	    ($LWL85m,$LWL85m_dN,$LWL85m_dS)=('-','-','-');
	    next;
	}

	if($LWL85m_dS!=0){
	    $LWL85m=round($LWL85m_dN/$LWL85m_dS,4);
	} else {
	    $LWL85m='-';
	}
    }
    if(/^LPB93:/){
	my $line=$_;
	if($line=~/dS =\s+([\d\.]+) dN =\s+([\d\.]+)/){
	    ($LPB93_dN,$LPB93_dS)=($2,$1);
	} else {
	    ($LPB93,$LPB93_dN,$LPB93_dS)=('-','-','-');
	    next;
	}

	if($LPB93_dS!=0){
	    $LPB93=round($LPB93_dN/$LPB93_dS,4);
	} else {
	    $LPB93='-';
	}
    }
}
print OUT "method : dN/dS  dN     dS\n";
print OUT "NG86   : $NG86 $NG86_dN $NG86_dS\n";
print OUT "YN00   : $YN00 $YN00_dN $YN00_dS\n";
print OUT "LWL85  : $LWL85 $LWL85_dN $LWL85_dS\n";
print OUT "LWL85m : $LWL85m $LWL85m_dN $LWL85m_dS\n";
print OUT "LPB93  : $LPB93 $LPB93_dN $LPB93_dS\n";

close IN;
close OUT;

sub round {
    my ($val,$col)=@_;
    my $r=10**$col;
    my $a=($val>0) ? 0.5 : -0.5;
    return int($val*$r+$a)/$r;
}
