#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i);
&getopts('hi:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a file containing RBH pairs (*.rbh)

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $rbhfile = $opt_i or die $usage;

# Read input files
my %list = read_rbh_file( $rbhfile );
my $numList = keys %list;

# Calculate and output difference
my $numSameName = 0;
foreach my $genePair (sort keys %list){
    my ($g1, $g2) = split("\t", $genePair);
    ++$numSameName if($g1 eq $g2);
}
print "No. of ortholog pairs  : " . $numList . "\n"
    . "No. of same name pairs : " . $numSameName . "\n"
    . "% of same name pairs   : " . 100*$numSameName/$numList . "\n";


sub read_rbh_file {
    my $file = shift;
    my %list;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split(" ", $_);
	if(@cols>=2){
	    my ($id1, $id2, $score) = @cols;
	    $list{$id1."\t".$id2} = 1;
	}
    }
    close IN;
    return %list;
}


