#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_o);
&getopts('hi:j:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a file containing RBH pairs (*.rbh)
# -j   another file containing RBH pairs (*.rbh)
# -o   output file

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $rbhfile1 = $opt_i or die $usage;
my $rbhfile2 = $opt_j or die $usage;
my $outfile  = $opt_o or die $usage;

# Read input files
my %list_1 = read_rbh_file( $rbhfile1 );
my %list_2 = read_rbh_file( $rbhfile2 );

# Calculate and output difference
diff( \%list_1, \%list_2, $outfile );


sub read_rbh_file {
    my $file = shift;
    my %list;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split(" ", $_);
	if(@cols>=2){
	    my ($id1, $id2, $score) = @cols;
	    $list{$id1."\t".$id2} = 1;
	}
    }
    close IN;
    return %list;
}


sub diff {
    my $list_1  = shift;
    my $list_2  = shift;
    my $outfile = shift;
    open(OUT, ">$outfile") or die "Can't open $outfile to write.";
    print OUT "In set 1 but not in set 2:" . "\n";
    print STDERR "In set 1 but not in set 2:" . "\n";
    my $cnt_1 = 0;
    foreach my $gp (sort keys %$list_1){
	++$cnt_1 unless( $$list_2{$gp} );
	print OUT $gp . "\n" unless( $$list_2{$gp} );
	print STDERR $gp . "\n" unless( $$list_2{$gp} );
    }
    print OUT $cnt_1 . " gene pairs." . "\n";
    print STDERR $cnt_1 . " gene pairs." . "\n";
    print OUT "\n" . "In set 2 but not in set 1:" . "\n";
    print STDERR "\n" . "In set 2 but not in set 1:" . "\n";
    my $cnt_2 = 0;
    foreach my $gp (sort keys %$list_2){
	++$cnt_2 unless ( $$list_1{$gp} );
	print OUT $gp . "\n" unless ( $$list_1{$gp} );
	print STDERR $gp . "\n" unless ( $$list_1{$gp} );
    }
    print OUT $cnt_2 . " gene pairs." . "\n";
    print STDERR $cnt_2 . " gene pairs." . "\n";
    close OUT;
}
