#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_s $opt_o);
&getopts('hi:s:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a file containing RBH pairs (*.rbh)
# -s   score cut-off
# -o   output file

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
die $usage if !defined $opt_s;
my $rbhFile = $opt_i or die $usage;
my $scr_cut = $opt_s;
my $outFile = $opt_o or die $usage;

# Read input files
my %rbhList = read_rbh_file( $rbhFile, $scr_cut );

# Output results
open(OUT, ">$outFile") or die "Can't open $outFile to write.";
foreach my $id_pair (sort keys %rbhList){
    my $score = $rbhList{$id_pair};
    print OUT $id_pair . "\t" . $score . "\n";
}
close OUT;



sub read_rbh_file {
    my $file    = shift;
    my $scr_cut = shift;
    my %list;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	my @cols = split(" ", $_);
	if(@cols==3){
	    my ($id1, $id2, $score) = @cols;
	    $list{$id1."\t".$id2} = $score if $score >= $scr_cut;
	}
    }
    close IN;
    return %list;
}



