#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_c $opt_m $opt_o);
&getopts('hi:c:m:M:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i    alignment file (srspair format)
# -c    list of cDNA sequence files
# -m    genetic code (either '0' -normal or '1' -mammalian mitochondria)
# -o    alignment file (nuc format)

## Others:
# -h print help
_EOH_
    ;

die $usage if $opt_h;

my $infile=$opt_i or die $usage;
my $dnafile=$opt_c or die $usage;
die $usage if !defined($opt_m);
my $flg_m=$opt_m; 
my $outfile=$opt_o or die $usage;
open(OUT,">$outfile") or die "Can't open $outfile to write.";

open(DNA,$dnafile) or die "Can't open $dnafile to read.";
my $id1=<DNA>;
chomp $id1;
$id1=~s/>//;
my $dna1=<DNA>;
chomp $dna1;
my $id2=<DNA>;
chomp $id2;
$id2=~s/>//;
my $dna2=<DNA>;
chomp $dna2;
close DNA;

my ($pep1, $pep2) = read_needle_file( $id1, $id2, $infile );
my ($sid1, $sid2) = ($id1, $id2);
$sid1=substr($id1,0,13) if length($id1)>13;
$sid2=substr($id2,0,13) if length($id2)>13;

my $p1=$pep1;
$p1=~s/\-//g;
my $p2=$pep2;
$p2=~s/\-//g;
if($flg_m){
    $dna1=matchM($dna1,$p1,$id1);
    $dna2=matchM($dna2,$p2,$id2);
} else {
    $dna1=match($dna1,$p1,$id1);
    $dna2=match($dna2,$p2,$id2);
}
if( ($dna1 eq 'N/A') or ($dna2 eq 'N/A') ){
    print 'N/A' . "\n"; exit;
}

my $len=length($pep1)*3;
print OUT "\t2\t$len\tI\n\n";
print OUT "$id1\n$id2\n";
my @dna1=split("|",$dna1);
my @dna2=split("|",$dna2);

my ($s1,$s2);
for(my $i=0;$i<length($pep1);++$i){
    if($i%20==0){
	if($i){
	    my @s1=split("|",$s1);
	    my @s2=split("|",$s2);
	    for(my $j=0;$j<length($s1);++$j){
		$s2[$j]='.' if ($s1[$j] eq $s2[$j]) and !($s1[$j] eq ' ');
	    }
	    $s1=join("",@s1);
	    $s2=join("",@s2);
	    print OUT $s1,"\n";
	    print OUT $s2,"\n";
	    undef $s1;
	    undef $s2;
	}
	print OUT $i*3+1,"\n" if $i%20==0;
    }

    my $aa1=substr($pep1,$i,1);
    my $aa2=substr($pep2,$i,1);
    if($aa1 eq '-'){
	$s1.=' ' if defined $s1;
	$s1.='---';
    } else {
	$s1.=' ' if defined $s1;
	$s1.=shift(@dna1);
	$s1.=shift(@dna1);
	$s1.=shift(@dna1);
    }
    if($aa2 eq '-'){
	$s2.=' ' if defined $s2;
	$s2.='---';
    } else {
	$s2.=' ' if defined $s2;
	$s2.=shift(@dna2);
	$s2.=shift(@dna2);
	$s2.=shift(@dna2);
    }

    if($i==length($pep1)-1){
	if($i){
	    my @s1=split("|",$s1);
	    my @s2=split("|",$s2);
	    for(my $j=0;$j<length($s1);++$j){
		$s2[$j]='.' if ($s1[$j] eq $s2[$j]) and !($s1[$j] eq ' ');
	    }
	    $s1=join("",@s1);
	    $s2=join("",@s2);
	    print OUT $s1,"\n";
	    print OUT $s2,"\n";
	    undef $s1;
	    undef $s2;
	}
    }
}
close OUT;
print 0 . "\n";


sub match {
    my ($dna,$pep)=@_;
    my $pep_len=length($pep);
    my $dna_len=length($dna);
    my $flgF=0;
    while(-1){
	if($flgF){
	    $dna=substr($dna,1,length($dna)-1);
	    # print length($dna),"\n";
	}
	$flgF=1;

	if(length($dna)<$pep_len*3){
	    print STDERR "Can't match peptide and DNA sequences:" . "\n";
	    return 'N/A';
	}
	my $sub_dna=substr($dna,0,$pep_len*3);
	my $sub_pep=dna2pep($sub_dna);
	next if !defined($sub_dna) or !defined($sub_pep);
	if($sub_pep eq $pep){
	    return $sub_dna;
	}
    }
}

sub matchM {
    my ($dna,$pep)=@_;
    my $pep_len=length($pep);
    my $dna_len=length($dna);
    my $flgF=0;
    while(-1){
	if($flgF){
	    $dna=substr($dna,1,length($dna)-1);
	    # print length($dna),"\n";
	}
	$flgF=1;

	if(length($dna)<$pep_len*3){
	    print STDERR "Can't match peptide and DNA sequences:" . "\n";
	    return 'N/A';
	}
	my $sub_dna=substr($dna,0,$pep_len*3);
	my $sub_pep=dna2pepM($sub_dna);
	next if !defined($sub_dna) or !defined($sub_pep);
	if($sub_pep eq $pep){
	    return $sub_dna;
	}
    }
}

sub dna2pep {
    my ($dna)=@_;
    my $pep;
    die "length of DNA sequences is not appropriate." unless length($dna)%3==0;
    my $pep_len=length($dna)/3;
    for(my $i=0;$i<$pep_len;++$i){
	my $codon=substr($dna,$i*3,3);
	my $aa=codon2aa($codon);
	last if $aa eq '.';
	$pep.=$aa;
    }
    return $pep;
}

sub dna2pepM {
    my ($dna)=@_;
    my $pep;
    die "length of DNA sequences is not appropriate." unless length($dna)%3==0;
    my $pep_len=length($dna)/3;
    for(my $i=0;$i<$pep_len;++$i){
	my $codon=substr($dna,$i*3,3);
	my $aa=codon2aaM($codon);
	last if $aa eq '.';
	$pep.=$aa;
    }
    return $pep;
}


sub codon2aa {
    my ($codon)=@_;
    die "length of a codon is not appropriate." unless length($codon)==3;
    my %codon2aa;
    $codon2aa{'TTT'}='F';
    $codon2aa{'TTC'}='F';
    $codon2aa{'TTA'}='L';
    $codon2aa{'TTG'}='L';

    $codon2aa{'TCT'}='S';
    $codon2aa{'TCC'}='S';
    $codon2aa{'TCA'}='S';
    $codon2aa{'TCG'}='S';

    $codon2aa{'TAT'}='Y';
    $codon2aa{'TAC'}='Y';
    $codon2aa{'TAA'}='.';
    $codon2aa{'TAG'}='.';

    $codon2aa{'TGT'}='C';
    $codon2aa{'TGC'}='C';
    $codon2aa{'TGA'}='.';
    $codon2aa{'TGG'}='W';

    $codon2aa{'CTT'}='L';
    $codon2aa{'CTC'}='L';
    $codon2aa{'CTA'}='L';
    $codon2aa{'CTG'}='L';

    $codon2aa{'CCT'}='P';
    $codon2aa{'CCC'}='P';
    $codon2aa{'CCA'}='P';
    $codon2aa{'CCG'}='P';

    $codon2aa{'CAT'}='H';
    $codon2aa{'CAC'}='H';
    $codon2aa{'CAA'}='Q';
    $codon2aa{'CAG'}='Q';

    $codon2aa{'CGT'}='R';
    $codon2aa{'CGC'}='R';
    $codon2aa{'CGA'}='R';
    $codon2aa{'CGG'}='R';

    $codon2aa{'ATT'}='I';
    $codon2aa{'ATC'}='I';
    $codon2aa{'ATA'}='I';
    $codon2aa{'ATG'}='M';

    $codon2aa{'ACT'}='T';
    $codon2aa{'ACC'}='T';
    $codon2aa{'ACA'}='T';
    $codon2aa{'ACG'}='T';

    $codon2aa{'AAT'}='N';
    $codon2aa{'AAC'}='N';
    $codon2aa{'AAA'}='K';
    $codon2aa{'AAG'}='K';

    $codon2aa{'AGT'}='S';
    $codon2aa{'AGC'}='S';
    $codon2aa{'AGA'}='R';
    $codon2aa{'AGG'}='R';

    $codon2aa{'GTT'}='V';
    $codon2aa{'GTC'}='V';
    $codon2aa{'GTA'}='V';
    $codon2aa{'GTG'}='V';

    $codon2aa{'GCT'}='A';
    $codon2aa{'GCC'}='A';
    $codon2aa{'GCA'}='A';
    $codon2aa{'GCG'}='A';

    $codon2aa{'GAT'}='D';
    $codon2aa{'GAC'}='D';
    $codon2aa{'GAA'}='E';
    $codon2aa{'GAG'}='E';

    $codon2aa{'GGT'}='G';
    $codon2aa{'GGC'}='G';
    $codon2aa{'GGA'}='G';
    $codon2aa{'GGG'}='G';

    return $codon2aa{$codon} if $codon2aa{$codon};
    die "there is not appropriate codon.";
}

sub codon2aaM {
    my ($codon)=@_;
    die "length of a codon is not appropriate." unless length($codon)==3;
    my %codon2aa;
    $codon2aa{'TTT'}='F';
    $codon2aa{'TTC'}='F';
    $codon2aa{'TTA'}='L';
    $codon2aa{'TTG'}='L';

    $codon2aa{'TCT'}='S';
    $codon2aa{'TCC'}='S';
    $codon2aa{'TCA'}='S';
    $codon2aa{'TCG'}='S';

    $codon2aa{'TAT'}='Y';
    $codon2aa{'TAC'}='Y';
    $codon2aa{'TAA'}='.';
    $codon2aa{'TAG'}='.';

    $codon2aa{'TGT'}='C';
    $codon2aa{'TGC'}='C';
    $codon2aa{'TGA'}='W';
    $codon2aa{'TGG'}='W';

    $codon2aa{'CTT'}='L';
    $codon2aa{'CTC'}='L';
    $codon2aa{'CTA'}='L';
    $codon2aa{'CTG'}='L';

    $codon2aa{'CCT'}='P';
    $codon2aa{'CCC'}='P';
    $codon2aa{'CCA'}='P';
    $codon2aa{'CCG'}='P';

    $codon2aa{'CAT'}='H';
    $codon2aa{'CAC'}='H';
    $codon2aa{'CAA'}='Q';
    $codon2aa{'CAG'}='Q';

    $codon2aa{'CGT'}='R';
    $codon2aa{'CGC'}='R';
    $codon2aa{'CGA'}='R';
    $codon2aa{'CGG'}='R';

    $codon2aa{'ATT'}='I';
    $codon2aa{'ATC'}='I';
    $codon2aa{'ATA'}='M';
    $codon2aa{'ATG'}='M';

    $codon2aa{'ACT'}='T';
    $codon2aa{'ACC'}='T';
    $codon2aa{'ACA'}='T';
    $codon2aa{'ACG'}='T';

    $codon2aa{'AAT'}='N';
    $codon2aa{'AAC'}='N';
    $codon2aa{'AAA'}='K';
    $codon2aa{'AAG'}='K';

    $codon2aa{'AGT'}='S';
    $codon2aa{'AGC'}='S';
    $codon2aa{'AGA'}='.';
    $codon2aa{'AGG'}='.';

    $codon2aa{'GTT'}='V';
    $codon2aa{'GTC'}='V';
    $codon2aa{'GTA'}='V';
    $codon2aa{'GTG'}='V';

    $codon2aa{'GCT'}='A';
    $codon2aa{'GCC'}='A';
    $codon2aa{'GCA'}='A';
    $codon2aa{'GCG'}='A';

    $codon2aa{'GAT'}='D';
    $codon2aa{'GAC'}='D';
    $codon2aa{'GAA'}='E';
    $codon2aa{'GAG'}='E';

    $codon2aa{'GGT'}='G';
    $codon2aa{'GGC'}='G';
    $codon2aa{'GGA'}='G';
    $codon2aa{'GGG'}='G';

    return $codon2aa{$codon} if $codon2aa{$codon};
    die "there is not appropriate codon.";
}


sub read_needle_file {
    my $g1   = shift;
    my $g2   = shift;
    my $file = shift;
    my ($aln1, $aln2);
    open(IN, $file) or die "Can't open $file to read.";
    my $cnt = 0;
    while(<IN>){
	chomp;
	next if( /^\#/ );
	my @cols = split(' ', $_);
	if( @cols == 4 ){
	    next unless( $cols[1] =~ /^\d+$/ );
	    next unless( $cols[3] =~ /^\d+$/ );
	    if( ++$cnt % 2 == 1 ){
		$aln1 .= $cols[2];
	    } else {
		$aln2 .= $cols[2];
	    }
	}
    }
    close IN;
    return ($aln1, $aln2);
}
