#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i);
&getopts('hi:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a file containing the statistics of the DPD clustering (*.cluster.stats)

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $statsFile = $opt_i or die $usage;

# Read cluster stats file
my %size2freq = read_stats_file( $statsFile );

# Calculate the average of synteny size
my $aveSize   = average_size( \%size2freq );

# Calculate the maximum of synteny size
my $maxSize   = maximum_size( \%size2freq );

# Print synteny statistics
print "\n"
    . 'Ave. of synteny size = ' . $aveSize . "\n"
    . 'Max. of synteny size = ' . $maxSize . "\n";

################################################### 
# subroutine - read_stats_file
#    IN  = fine name
#    OUT = a hash from 'cluster size' to 'frequency'
###################################################
sub read_stats_file {
    my $file = shift;
    my %size2freq;
    open(IN, $file) or die "Can't open $file to read.";
    my $flg  = 0;
    while(<IN>){
	chomp;
	my @cols = split("\t", $_);
	if( /Cluster size frequency:/ ){ $flg = 1; }
	if( $flg and (@cols==2) ){
	    my ($size, $freq) = @cols;
	    $size2freq{$size} = $freq;
	}
    }
    close IN;
    return %size2freq;
}


################################################### 
# subroutine - average_size
#    IN  = reference to a hash from 'cluster size' to 'frequency'
#    OUT = average of synteny size
###################################################
sub average_size {
    my $ref_size2freq      = shift;
    my ($sizeSum, $synSum) = (0,0);
    foreach my $size (sort keys %$ref_size2freq){
	my $freq = $$ref_size2freq{$size};
	if( $size == 1 ){ next; }
	$sizeSum += $freq * $size;
	$synSum  += $freq;
    }
    return $sizeSum / $synSum;
}


################################################### 
# subroutine - maximum_size
#    IN  = reference to a hash from 'cluster size' to 'frequency'
#    OUT = maximum of synteny size
###################################################
sub maximum_size {
    my $ref_size2freq = shift;
    my $maxSize       = 0;
    foreach my $size (sort keys %$ref_size2freq){
	if( $size > $maxSize ){ $maxSize = $size; }
    }
    return $maxSize;
}
