#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_j $opt_m);
&getopts('hi:j:m:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a file containing a list of values
# -j   a file containing a list of values
# -m   set the mode (0--two-sided, 1--greater, 2--less)

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
die $usage unless defined($opt_m);
my $file_1  = $opt_i or die $usage;
my $file_2  = $opt_j or die $usage;
my $mode    = $opt_m;
my $tmp_dir = "/tmp/";
my $alt     = 'alternative="t"';
$alt        = 'alternative="g"' if $mode==1;
$alt        = 'alternative="l"' if $mode==2;

# Execute R
my $Rfile   = $tmp_dir . 'wilcox.' . $$ . '.R';
my $tmp_out = $tmp_dir . 'wilcox.' . $$ . '.out';
open(R, ">$Rfile") or die "Can't open $Rfile to write.";
print R 'data1 <- read.table("' . $file_1 . '")'        . "\n"
    . 'data2 <- read.table("' . $file_2 . '")'          . "\n"
    . 'wtest <- wilcox.test(data1[,1], data2[,1], ' 
    . $alt . ', exact=FALSE)'                           . "\n"
    . 'out <- file("' . $tmp_out . '", "w")'            . "\n"
    . 'writeLines(paste(wtest$p.value), out, sep="\n")' . "\n"
    . 'close(out)'                                      . "\n";
close R;
`R CMD BATCH $Rfile`;
print `cat $tmp_out`;
`rm $Rfile`;
`rm $tmp_out`;
