#ifndef __INC_AGG_H__
#define __INC_AGG_H__

#define STEP_SIZE_AGG 0.01
#define TOL_AGG 1e-4
#define EPSABS_AGG 1e-3
#define ITER_AGG 100
#define NEARLY_ZERO_AGG 1e-100
#define HIST_BIN_AGG 50
#define PLOT_SAMPLES_AGG 10000
#define MIN_SD_AGG 0.01
#define MIN_SHAPE_AGG 0.1
#define M_AGG 4

#include <cmath>
#include <iostream>
#include <fstream>
#include <gsl/gsl_sf_psi.h>
#include <gsl/gsl_sf_gamma.h>
#include <gsl/gsl_multimin.h>
#include "param.h"
#include "pdf.h"
#include "histogram.h"
#include "util.h"

using namespace std;

class pdf_agg;
class param_agg : public param {
  friend class pdf_agg;
  double loc, sd_left, sd_right, shape;
  double sd;
 public:
  virtual ~param_agg(){}
  param_agg(){}
  param_agg( double d1, double d2, double d3, double d4 )
    { loc=d1; sd_left=d2; sd_right=d3; shape=d4; }
  param_agg( vector<double>& x );
  param_agg( vector<double>& x, vector<double>& w );
  void set_shape( double c ){ shape = c; }
  void show();
};


class pdf_agg : public pdf {
  param_agg* p;
 public:
  virtual ~pdf_agg(){}
  virtual void set_param( void* par ){ p = (param_agg*)par; }
  virtual void fit_ml( vector<double>& x );
  virtual void fit_ml( vector<double>& x, vector<double>& w );
  virtual double eval( double x );
  virtual double log_L( vector<double>& x );
  virtual double log_L( vector<double>& x, vector<double>& w );
  virtual double log_dL( vector<double>& x );
  virtual double log_dL( vector<double>& x, vector<double>& w );
  virtual double aic( vector<double>& x ){ return log_L(x)-M_AGG; }
  virtual double aic( vector<double>& x, vector<double>& w )
  { return log_L(x, w)-M_AGG; }
  virtual void print_sample_points( string file, double min, double max );
  virtual void plot( string name );
  virtual void plot( string name, vector<double>& x, size_t num_bin );
  virtual void plot( string name, vector<double>& x, vector<double>& w, 
		     size_t num_bin );
  virtual void plot( string name, histogram& h );
  virtual void plot( string name, vector<void*>& param_list );
  double cal_median( vector<double>& x );
  double cal_median( vector<double>& x, vector<double>& w );
  double cal_mode_hist( vector<double>& x );
  double cal_mode_hist( vector<double>& x, vector<double>& w );
  double cal_sd_left( vector<double>& x );
  double cal_sd_left( vector<double>& x, vector<double>& w );
  double cal_sd_right( vector<double>& x);
  double cal_sd_right( vector<double>& x, vector<double>& w );
  double cal_shape( vector<double>& x );
  double cal_shape( vector<double>& x, vector<double>& w );
};


typedef pair< vector<double>*, param_agg* > gsl_param_agg;
double agg_f( const gsl_vector* v, void* gsl_param );
void agg_df( const gsl_vector* v, void* gsl_param, gsl_vector* df );
void agg_fdf( const gsl_vector* v, void* gsl_param, 
	      double* f, gsl_vector* df );
typedef vector<void*> gsl_param_w_agg;
double w_agg_f( const gsl_vector* v, void* gsl_param );
void w_agg_df( const gsl_vector* v, void* gsl_param, gsl_vector* df );
void w_agg_fdf( const gsl_vector* v, void* gsl_param, 
		double* f, gsl_vector* df );

#endif // __INC_AGG_H__
