// *************************************************************************
//  c_hit_set.cpp
// *************************************************************************

#include "c_hit_set.h"
using namespace std;


// *************************************************************************
// Class c_hit_set
// Constructor
c_hit_set::c_hit_set( hit_set& hs12, hit_set& hs21 ){
  map<string, hit>::iterator it = hs12.begin();
  while( it != hs12.end() ){
    string IDpair = it->first;
    list<string> IDs = split(IDpair, " ");
    list<string>::iterator id_it = IDs.begin();
    string ID_1 = *id_it; ++id_it;
    string ID_2 = *id_it; ++id_it;
    string rev_IDpair = ID_2 + " " + ID_1;
    map<string, hit>::iterator it_21;
    if( (it_21 = hs21.find( rev_IDpair )) != hs21.end() ){
      double bit_12 = it->second.get_bit();
      double bit_21 = it_21->second.get_bit();
      double bit    = (bit_12+bit_21)/2.0;
      c_hit  c_hit( ID_1, ID_2, bit );
      insert( c_hit.get_IDpair(), c_hit );
    }
    ++it;
  }
  set_stats();
}


// *************************************************************************
// Class c_hit_set
// Calculate base statistics for the hit set
void c_hit_set::add( c_hit_set c_hs ){
  map<string, c_hit>::iterator it = c_hs.begin();
  while( it != c_hs.end() ){
    string id_pair = it->first;
    c_hit  ch      = it->second;
    if( !exist(id_pair) )
      insert( id_pair, ch );
    ++it;
  }
}


// *************************************************************************
// Class c_hit_set
// Calculate base statistics for the hit set
void c_hit_set::set_stats(){
  double min = 0.0;
  double max = 0.0;
  double ave = 0.0;
  double sum = 0.0;
  map<string, c_hit>::iterator it = begin();
  while( it != end() ){
    double bit = it->second.get_bit();
    if( sum == 0.0 ) { max = bit; min = bit; }
    sum += bit;
    if( bit > max ) { max = bit; }
    if( bit < min ) { min = bit; }
    ++it;
  }
  ave = sum / size();
  min_bit = min; 
  max_bit = max; 
  ave_bit = ave;
}


// *************************************************************************
// Class c_hit_set
// Insert new hit 
void c_hit_set::insert(string ip, c_hit h){
  IDpair2hit.erase( ip );
  IDpair2hit.insert( map<string, c_hit>::value_type( ip, h ) );
}


// *************************************************************************
// Class c_hit_set
// Erase hit specified by ID pair
void c_hit_set::erase( string ip ){
  IDpair2hit.erase( ip );
}


// *************************************************************************
// Class c_hit_set
// Return if a ID pair is existing in the hit set, or not
bool c_hit_set::exist( string ip ){
  if( IDpair2hit.find(ip) == IDpair2hit.end() ){ return false; }
  else{ return true; }
}


// *************************************************************************
// Class c_hit_set
// Return the bit score of the gene pair specified ID pair
double c_hit_set::get_bit( string ip ){
  map<string, c_hit>::iterator it = find( ip );
  if( it == end() ){
    cerr << "Error! : " << ip << " is not found." << endl;
    exit(-1);
  }
  return it->second.get_bit();
}


// *************************************************************************
// Class c_hit_set
// Print ID pairs existing in the hit set
void c_hit_set::print_IDpairs() {
  map<string, c_hit>::iterator it = IDpair2hit.begin();
  while( it != IDpair2hit.end() ){
    cout << it->first << endl;
    ++it;
  }
}


