#ifndef __INC_DPD_CLUSTER__
#define __INC_DPD_CLUSTER__

#include <string>
#include <vector>
#include <map>
#include <math.h>
#include "gene_set.h"
#include "c_hit_set.h"
#include "ortholog_set.h"
#include "position_2d_map.h"
#include "progress_bar.h"

class dpd_cluster {
  map<string, unsigned> gp2cid; // cid = cluster ID
  map< unsigned, list<string> > cid2gp_list;

  unsigned get_min_cid( string gp, position_2d_map& pos_2d_map, 
			double col_index, unsigned t_dpd, unsigned& next_cid );
  bool is_neighbor( string gp, list<string>& gp_list, 
		    position_2d_map& pos_2d_map, double col_index, 
		    unsigned t_dpd );
  void append( string gp, unsigned min_cid );
  void merge_cluster( string gp, position_2d_map& pos_2d_map, 
		      double col_index, unsigned t_dpd, unsigned min_cid );
  void change_cid( list<string>& gp_list, unsigned cid );
 public:
  dpd_cluster( ortholog_set& os, position_2d_map& pos_2d_map, 
	       double col_index, unsigned t_dpd );
  size_t size();
  size_t num_cluster();
  size_t num_isolated();
  double average_cluster_size();
  double sd_cluster_size();
  map<size_t, size_t> size_frequency();
  void print( gene_set& gs1, gene_set& gs2, string outfile );
  void print_iso( c_hit_set& c_hs, gene_set& gs1, gene_set& gs2, 
		  string outfile );
  void print_non_iso( c_hit_set& c_hs, gene_set& gs1, gene_set& gs2, 
		      string outfile );
  void print_details( c_hit_set& c_hs, gene_set& gs1, gene_set& gs2, 
		      string outfile );
  void print_stats( string outfile );
};


#endif // __INC_DPD_CLUSTER__
