// *************************************************************************
//  dpd_clustering.cpp
// *************************************************************************

#include "dpd_clustering.h"

// *************************************************************************
// main function.
int main( int argc, char* argv[] ){
  // Get options
  cerr << endl;
  cerr << "Getting options...";
  dpd_options opt( argc, argv );
  string opt_file = opt.get_opt_file();
  opt.print();
  opt.print( opt_file ); 

  // Record start time
  time_t sttTime = time(0);

  // Load hit files
  cerr << "Loading input files..." << endl;
  time_t baseTime   = time(0);
  gene_set gene_set_1( opt.get_genefile_1() );
  gene_set gene_set_2( opt.get_genefile_2() );
  ortholog_set rbh_set_12( opt.get_rbh_file() );
  cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Compute position matrix 
  cerr << "Computing position matrix..." << endl;
  baseTime = time(0);
  position_map pos_map_1( gene_set_1 );
  cerr << pos_map_1.gene_size() << " genes in "
       << opt.get_orgname_in1() << " were registered in our position map."
       << endl;
  position_map pos_map_2( gene_set_2 );
  cerr << pos_map_2.gene_size() << " genes in "
       << opt.get_orgname_in2() << " were registered in our position map."
       << endl;
  c_hit_set c_hit_set_12 = rbh_set_12.get_c_hit_set();
  position_2d_map pos_2d_map_12( pos_map_1, pos_map_2, c_hit_set_12 );
  cerr << pos_2d_map_12.hit_size() << " RBHs were registered "
       << "in our 2D position map." << endl;
  cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Perform DPD clustering
  cerr << "Performing DPD clustering..." << endl;
  baseTime = time(0);
  double col_index = opt.get_col_index();
  unsigned t_dpd   = opt.get_t_dpd();
  dpd_cluster dpd_cluster_12( rbh_set_12, pos_2d_map_12, col_index, t_dpd );
  cerr << dpd_cluster_12.size() << " RBH pairs were clustered." << endl;
  cerr << dpd_cluster_12.num_cluster() << " clusters were found." << endl;
  cerr << dpd_cluster_12.num_isolated() << " isolated RBH pairs" << endl;
  cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Print the results
  cerr << "Printing the results..." << endl;
  baseTime = time(0);
  string clus_file    = opt.get_clus_file();
  string iso_file     = opt.get_iso_file();
  string non_iso_file = opt.get_non_iso_file();
  string detail_file  = opt.get_detail_file();
  string stat_file    = opt.get_stat_file();
  dpd_cluster_12.print( gene_set_1, gene_set_2, clus_file );
  dpd_cluster_12.print_iso( c_hit_set_12, gene_set_1, gene_set_2, iso_file );
  dpd_cluster_12.print_non_iso( c_hit_set_12, gene_set_1, gene_set_2, 
				non_iso_file );
  dpd_cluster_12.print_details( c_hit_set_12, gene_set_1, gene_set_2, 
				detail_file );
  dpd_cluster_12.print_stats( stat_file );
  cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;
  

  cerr << "Total time : " << time(0) - sttTime << " seconds" << endl;
  return 0;
}

