// *************************************************************************
//  dpd_options.cpp
// *************************************************************************

#include "dpd_options.h"
using namespace std;

// *************************************************************************
// Class dpd_options
// Constructor
// Parse command line and get options
dpd_options::dpd_options( int argc, char* argv[] ){
  char opt;
  int opt_idx;
  bool flg_I = false;
  bool flg_J = false;
  bool flg_P = false;
  bool flg_G = false;
  bool flg_H = false;

  // Set default parameters
  col_index = DEFAULT_COL_INDEX;
  t_dpd     = DEFAULT_T_DPD;

  string usage;
  usage = (string)"\n"
    + "********************************************************"  + "\n"
    + "Description:"                                              + "\n"
    + "  DPD clustering"                                          + "\n"
                                                                  + "\n"
    + "Options:"                                                  + "\n"
    + "  -i  set organism name (1)"                               + "\n"
    + "  -j  set organism name (2)"                               + "\n"
    + "  -p  set input file name"                                 + "\n"
    + "  -g  set path to directory containing gene files"         + "\n"
    + "  -l  set collinearity index"                                   + "\n"
    + "       (default = " + double2string( DEFAULT_COL_INDEX ) + ")"  + "\n"
    + "  -d  set threshold value for DPD value"                   + "\n"
    + "       (default = " + int2string( DEFAULT_T_DPD ) + ")"    + "\n"
    + "  -h  print this message"                                  + "\n"
    + "********************************************************"  + "\n";
    
  opt_idx = 0;
  while( -1 ){
    opt = getopt( argc, argv, "i:j:p:g:l:d:h" );
    opt_idx++;
    if( opt == -1 ){
      break;
    }

    switch( opt ){
    case 'i':
      orgname_in1 = (string)argv[++opt_idx];
      flg_I = true;
      break;
    case 'j':
      orgname_in2 = (string)argv[++opt_idx];
      flg_J = true;
      break;
    case 'p':
      rbh_file = (string)argv[++opt_idx];
      flg_P = true;
      break;
    case 'g':
      gene_dir = (string)argv[++opt_idx];
      flg_G = true;
      break;
    case 'l':
      col_index = atof( argv[++opt_idx] );
      break;
    case 'd':
      t_dpd = atoi( argv[++opt_idx] );
      break;
    case 'h':
      flg_H = true;
      break;
    case '?':
    default:
      cerr << "Unknown option " << opt << endl;
      exit( -1 );
    }
  }
  if( flg_H || !flg_I || !flg_J || !flg_P || !flg_G ){
    cerr << usage << endl;
    exit(1);
  }
  set_file_name();
}


// *************************************************************************
// Class dpd_options
// Set file names
void dpd_options::set_file_name() {
  genefile_1       = gene_dir + "/" + orgname_in1 + ".genes";
  genefile_2       = gene_dir + "/" + orgname_in2 + ".genes";
  opt_file         = rbh_file + ".options";
  clus_file        = rbh_file + ".cluster";
  iso_file         = rbh_file + ".isolated";
  non_iso_file     = rbh_file + ".non_isolated";
  detail_file      = rbh_file + ".details";
  stat_file        = clus_file + ".stats";
}


// *************************************************************************
// Class dpd_options
// Print options
void dpd_options::print() {
  string options = (string)"\n"
    + "********************************************************"   + "\n"
    + "Options:"                                                   + "\n"
    + "IN organism (1)      : " + orgname_in1                      + "\n"
    + "IN organism (2)      : " + orgname_in2                      + "\n"
    + "Input file           : " + rbh_file                         + "\n"
    + "Collinearity index   : " + double2string( col_index )       + "\n"
    + "Threshold for DPD    : " + int2string( t_dpd )              + "\n"
    + "********************************************************"   + "\n";
  cerr << options << endl;
}


// *************************************************************************
// Class dpd_options
// Print options
void dpd_options::print( string outfile ) {
  ofstream ofs( outfile.c_str() );
  string options = (string)"\n"
    + "********************************************************"   + "\n"
    + "Options:"                                                   + "\n"
    + "IN organism (1)      : " + orgname_in1                      + "\n"
    + "IN organism (2)      : " + orgname_in2                      + "\n"
    + "Input file           : " + rbh_file                         + "\n"
    + "Collinearity index   : " + double2string( col_index )       + "\n"
    + "Threshold for DPD    : " + int2string( t_dpd )              + "\n"
    + "********************************************************"   + "\n";
  ofs << options << endl;
  ofs.close();
}
