// *************************************************************************
//  draw_wnnso_matrix.cpp
// *************************************************************************

#include <ctime>
#include "draw_wnnso_matrix.h"
#include "draw_wnnso_options.h"
#include "seed_matrix.h"

// *************************************************************************
// main function.
int main( int argc, char* argv[] ){
  // Get options
  cerr << endl;
  cerr << "Getting options...";
  draw_wnnso_options opt( argc, argv );
  opt.print();

  // Record start time
  time_t sttTime = time(0);

  // Load seed position file
  cerr << "Loading seed position file..." << endl;
  time_t baseTime = time(0);
  string pos_file = opt.get_pos_file();
  seed_matrix seed_mat( pos_file );
  cerr << seed_mat.get_n_cols() << "*" << seed_mat.get_n_rows()
       << " matrix was created." << endl
       << seed_mat.num_seeds() << " seeds exist in the matrix." << endl;
  cerr << "done ("
       << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Calculate the WNNSO value of the cells in the matrix
  cerr << "Calculating thw WNNSO values..." << endl;
  baseTime = time(0);
  double sigma = opt.get_sigma();
  seed_mat.calc_wnnso( sigma );
  cerr << "done ("
       << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Draw heatmap
  cerr << "Drawing heatmaps..." << endl;
  baseTime = time(0);
  string Rdir = opt.get_Rdir();
  seed_mat.draw( pos_file, Rdir );
  cerr << "done ("
       << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Report total time required for this program
  cerr << "Total time : " << time(0) - sttTime << " seconds" << endl;

  return 0;
}
