// *************************************************************************
//  draw_wnnso_options.cpp
// *************************************************************************

#include "draw_wnnso_options.h"
using namespace std;

// *************************************************************************
// Class draw_wnnso_options
// Constructor
// Parse command line and get options
draw_wnnso_options::draw_wnnso_options( int argc, char* argv[] ){
  char opt;
  int opt_idx;
  bool flg_I = false;
  bool flg_S = false;
  bool flg_R = false;
  bool flg_H = false;

  string usage;
  usage = (string)"\n"
    + "********************************************************"  + "\n"
    + "Description:"                                              + "\n"
    + "  draw_wnnso_matrix"                                       + "\n"
    + "    --  Draw WNNSO matrix"                                 + "\n"
                                                                  + "\n"
    + "Options:"                                                  + "\n"
    + "  -i  set the file containing seed positions"              + "\n"
    + "  -s  set sigma parameter"                                 + "\n"
    + "  -r  set the directory containign R scripts"              + "\n"
    + "  -h  print this message"                                  + "\n"
    + "********************************************************"  + "\n";

  opt_idx = 0;
  while( -1 ){
    opt = getopt( argc, argv, "i:s:r:h" );
    opt_idx++;
    if( opt == -1 ){
      break;
    }
    switch( opt ){
    case 'i':
      pos_file = (string)argv[++opt_idx];
      flg_I = true;
      break;
    case 's':
      sigma = atof( argv[++opt_idx] );
      flg_S = true;
      break;
    case 'r':
      Rdir = (string)argv[++opt_idx];
      flg_R = true;
      break;
    case 'h':
      flg_H = true;
      break;
    case '?':
    default:
      cerr << "Unknown option " << opt << endl;
      exit( -1 );
    }
  }
  if( flg_H || !flg_I || !flg_S || !flg_R ){
    cerr << usage << endl;
    exit(1);
  }
}


// *************************************************************************
// Class draw_wnnso_options
// Print options
void draw_wnnso_options::print() {
  string options = (string)"\n"
    + "********************************************************"  + "\n"
    + "Options:"                                                  + "\n"
    + "Position file       : " + pos_file                         + "\n"
    + "Sigma               : " + double2string( sigma )           + "\n"
    + "R directory         : " + Rdir                             + "\n"
    + "********************************************************"  + "\n";
  cerr << options << endl;
}
