// *************************************************************************
//  group_set.cpp
// *************************************************************************

#include "group_set.h"


// *************************************************************************
// Class group_set
// Constructor
void group_set::load( string in_file ){
  try {
    ifstream ifs( in_file.c_str() );
    if( ifs.fail() )
      throw (string)"File open error: " + in_file.c_str();
    string str;
    while( getline(ifs, str) ){
      list<string> cols = split( str, "\t" );
      if( cols.size() != 3 )
	throw "Inappropriate file format: " + str;
      list<string>::iterator it = cols.begin();
      unsigned group_id = atoi( it->c_str() ); ++it;
      string   str1     = *it; ++it;
      string   str2     = *it; ++it;
      group    gr( str1, str2 );
      insert( group_id, gr );
    }
    ifs.close();
  } catch (string err_msg){
    cerr << err_msg << endl;
    exit(-1);
  }
}


// *************************************************************************
// Class group_set
// Extract seed orthologs
ortholog_set group_set::extract_seed_set(){
  ortholog_set os;
  map<unsigned, group>::iterator it = begin();
  while( it != end() ){
    group gr = it->second;
    c_hit_set c_hs = gr.get_all_gene_pairs();
    if( c_hs.size() == 1 ){
      string id_pair = gr.first_pair();
      double dummy_score = 0.0;
      os.insert( id_pair, dummy_score );
    }
    ++it;
  }
  return os;
}


// *************************************************************************
// Class group_set
// Extract positional orthologs
ortholog_set group_set::extract_ortholog_set( position_2d_map& pos_2d_map, 
					      ortholog_set& seeds, 
					      double col_index, 
					      double sds, double t_dpd ){
  ortholog_set os;
  map<unsigned, group>::iterator it = begin();
  while( it != end() ){
    group gr = it->second;
    c_hit_set c_hs = gr.get_all_gene_pairs();
    oasys_hit_set o_hs; 
    o_hs.set( c_hs, pos_2d_map, seeds, col_index, sds, t_dpd, false );
    map<string, oasys_hit>::iterator o_hs_it = o_hs.begin();
    while( o_hs_it != o_hs.end() ){
      string id_pair = o_hs_it->first;
      double wnnso   = o_hs_it->second.get_wnnso();
      w_insert( id_pair, wnnso );
      ++o_hs_it;
    }
    ortholog_set tmp_os = o_hs.get_wnnso_rbh();
    map<string, double>::iterator os_it = tmp_os.begin();
    while( os_it != tmp_os.end() ){
      string id_pair = os_it->first;
      double wnnso   = os_it->second;
      if( wnnso > 0 || seeds.exist( id_pair ) ) 
	os.insert( id_pair, wnnso );
      ++os_it;
    }
    ++it;
  }
  return os;
}


// *************************************************************************
// Class group_set
// Insert a new element
void group_set::insert( unsigned id, group gr ){
  erase( id );
  id2group.insert( map<unsigned, group>::value_type( id, gr ) );
}


// *************************************************************************
// Class group_set
// Insert a new element
void group_set::w_insert( string id_pair, double wnnso ){
  w_erase( id_pair );
  ip2wnnso.insert( map<string, double>::value_type( id_pair, wnnso ) );
}


// *************************************************************************
// Class group_set
// Return all gene pairs
c_hit_set group_set::get_all_gene_pairs(){
  c_hit_set c_hs;
  map<unsigned, group>::iterator it = begin();
  while( it != end() ){
    group g = it->second;
    c_hit_set tmp_c_hs = g.get_all_gene_pairs();
    c_hs.add( tmp_c_hs );
    ++it;
  }
  return c_hs;
}


// *************************************************************************
// Class group_set
// Print info
void group_set::print_info( string outfile ){
  ofstream ofs( outfile.c_str() );
  map<string, double>::iterator it = w_begin();
  while( it != w_end() ){
    string id_pair = it->first;
    double wnnso   = it->second;
    list<string> id_list = split(id_pair, (string)" ");
    list<string>::iterator id_it = id_list.begin();
    string id1 = *id_it; ++id_it;
    string id2 = *id_it; ++id_it;
    ofs << id1 << "\t" << id2 << "\t" << wnnso << endl;
    ++it;
  }
  ofs.close();
}
