#ifndef __INC_HISTOGRAM_H__
#define __INC_HISTOGRAM_H__
#define SMALL_VALUE_HISTOGRAM 0.00001

#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <gsl/gsl_histogram.h>

using namespace std;
typedef pair<double, double> point;

class histogram {
  gsl_histogram* h;
  size_t hist_size;
  double min, max, obs_max;
  
  double find_max( vector<double>& val_list );
  double find_min( vector<double>& val_list );
public:
  histogram( size_t n, vector<double>& val_list );
  histogram( size_t n, vector<double>& val_list, vector<double>& w_list );

  size_t size(){ return hist_size; }
  double get_min(){ return min; }
  double get_max(){ return max; }
  double get_val( double x ){ 
    size_t i; gsl_histogram_find(h, x, &i); 
    return gsl_histogram_get(h, i);
  }
  double get( size_t i ){ return gsl_histogram_get(h, i); }
  double get_max_val(){ return gsl_histogram_max_val( h ); }
  double get_mode();
  vector< point > get_points();
  vector< point > get_points( unsigned stt, unsigned end );

  void show(){ gsl_histogram_fprintf( stdout, h, "%g", "%g" ); }
  void print( string outfile, bool flg_prob = false );
  void print( string outfile, unsigned stt, unsigned end, 
	      bool flg_prob = false );
  void draw( string name, bool flg_prob = false );
  void draw( string name, unsigned stt, unsigned end, 
	     bool flg_prob = false );
  void draw( histogram& h1, string name, bool flg_prob = false );
  void draw( histogram& h2, string name, unsigned stt, unsigned end, 
	     bool flg_prob = false );

  ~histogram(){ gsl_histogram_free( h ); } 
};


#endif // __INC_HISTOGRAM_H__
