// *************************************************************************
//  hit_set.cpp
// *************************************************************************

#include "hit_set.h"
using namespace std;


// *************************************************************************
// Class hit_set
// Constructor
hit_set::hit_set( string in_file ){
  try {
    ifstream ifs( in_file.c_str() );
    if( ifs.fail() )
      throw (string)"File open error: " + in_file.c_str();
    string str;
    while( getline(ifs, str) ){
      list<string> cols = split( str, "\t" );
      if( cols.size() != 3 )
	throw (string)"Inappropriate file format: " + str;
      list<string>::iterator it = cols.begin();
      string id1 = *it; ++it;
      string id2 = *it; ++it;
      double bit = atof(it->c_str()); ++it;
      insert( id1, id2, bit );
    }
    ifs.close();
  } catch ( string err_msg ){
    cerr << err_msg << endl; 
    exit(-1);
  }
}


// *************************************************************************
// Class hit_set
// Insert new hit 
void hit_set::insert(string ip, hit h){
  IDpair2hit.erase( ip );
  IDpair2hit.insert( map<string, hit>::value_type( ip, h ) );
}


// *************************************************************************
// Class hit_set
// Insert new hit 
void hit_set::insert(string ip, double score){
  hit h(ip, score);
  insert(ip, h);
}


// *************************************************************************
// Class hit_set
// Insert new hit 
void hit_set::insert(string id1, string id2, double score){
  hit h(id1, id2, score);
  insert( h.get_IDpair(), h );
}



// *************************************************************************
// Class hit_set
// Erase hit specified by ID pair
void hit_set::erase( string ip ){
  IDpair2hit.erase( ip );
}


// *************************************************************************
// Class hit_set
// Return if a ID pair is existing in the hit set, or not
bool hit_set::exist( string ip ){
  if( IDpair2hit.find(ip) == IDpair2hit.end() ){ return false; }
  else{ return true; }
}


// *************************************************************************
// Class hit_set
// Print ID pairs existing in the hit set
void hit_set::print_IDpairs() {
  map<string, hit>::iterator it = IDpair2hit.begin();
  while( it != IDpair2hit.end() ){
    cout << it->first << endl;
    ++it;
  }
}



