// *************************************************************************
//  make_wnnso_options.cpp
// *************************************************************************

#include "make_wnnso_options.h"
using namespace std;

// *************************************************************************
// Class make_wnnso_options
// Constructor
// Parse command line and get options
make_wnnso_options::make_wnnso_options( int argc, char* argv[] ){
  char opt;
  int opt_idx;
  bool flg_S = false;
  bool flg_C = false;
  bool flg_R = false;
  bool flg_O = false;
  bool flg_H = false;

  string usage;
  usage = (string)"\n"
    + "********************************************************"  + "\n"
    + "Description:"                                              + "\n"
    + "  make_wnnso_matrix"                                       + "\n"
    + "    --  Make input file for draw_wnnso_matrix"             + "\n"
                                                                  + "\n"
    + "Options:"                                                  + "\n"
    + "  -s  set the position of seeds"                           + "\n"
    + "       e.g. -s [1,2,1][4,8,-1]"                            + "\n"
    + "  -c  set the number of columns"                           + "\n"
    + "  -r  set the number of rows"                              + "\n"
    + "  -o  set the output file name"                            + "\n"
    + "  -h  print this message"                                  + "\n"
    + "********************************************************"  + "\n";

  opt_idx = 0;
  while( -1 ){
    opt = getopt( argc, argv, "s:c:r:o:h" );
    opt_idx++;
    if( opt == -1 ){
      break;
    }
    switch( opt ){
    case 's':
      seed_pos = (string)argv[++opt_idx];
      flg_S = true;
      break;
    case 'c':
      n_cols = atoi( argv[++opt_idx] );
      flg_C = true;
      break;
    case 'r':
      n_rows = atoi( argv[++opt_idx] );
      flg_R = true;
      break;
    case 'o':
      out_file = (string)argv[++opt_idx];
      flg_O = true;
      break;
    case 'h':
      flg_H = true;
      break;
    case '?':
    default:
      cerr << "Unknown option " << opt << endl;
      exit( -1 );
    }
  }
  if( flg_H || !flg_S || !flg_C || !flg_R || !flg_O ){
    cerr << usage << endl;
    exit(1);
  }
}


// *************************************************************************
// Class make_wnnso_options
// Print options
void make_wnnso_options::print() {
  string options = (string)"\n"
    + "********************************************************"  + "\n"
    + "Options:"                                                  + "\n"
    + "Seed position       : " + seed_pos                         + "\n"
    + "Number of columns   : " + int2string( (int)n_cols )      + "\n"
    + "Number of rows      : " + int2string( (int)n_rows )      + "\n"
    + "Output file         : " + out_file                         + "\n"
    + "********************************************************"  + "\n";
  cerr << options << endl;
}
