#ifndef __INC_MODEL_H__
#define __INC_MODEL_H__

#define HIST_BIN_MODEL 50
#define PLOT_SAMPLE_MODEL 10000
#define SLOPE_SCORE 0.00001
#define INTEGRAL_PARTITION 100000

#include <vector>
#include <cmath>
#include "ogg.h"
#include "agg.h"

using namespace std;

class model {
  param_ogg p_ogg;
  pdf_ogg   ogg;
  param_agg p_agg;
  pdf_agg   agg;
  bool      flg_model; // 0 means the ogg model, 1 means the agg model
 public:
  model( vector<double>& x, vector<double>& w );
  double eval( double x );
  void draw_pdf( string name, vector<double>& x, vector<double>& w );
};

double calc_score( model& m1, model& m0, double x );
double calc_mi_score( model& m1, model& m0, double x, double peak_val );
double calc_certainity( vector<double>& x, model& m1, model& m0 );
double calc_J_divergence( model& m1, model& m0, double min, double max );
void draw_histograms( string name, vector<double>& x, 
		      vector<double>& w0, vector<double>& w1 );
void draw_histogram( string name, vector<double>& x, vector<double>& w );
void draw_score_function( string name, model& m1, model& m0, 
			  double min, double max, 
			  size_t num_bin=PLOT_SAMPLE_MODEL );
void draw_mi_score_function( string name, model& m1, model& m0, 
			     double min, double max, double peak_val, 
			     size_t num_bin=PLOT_SAMPLE_MODEL );
void draw_two_score_function( string name, model& m1, model& m0, 
			      double min, double max, double peak_val, 
			      size_t num_bin=PLOT_SAMPLE_MODEL );
void print_sample_points( string file, model& m1, model& m0, 
			  double min, double max, size_t num_bin );
void print_sample_points_mi( string file, model& m1, model& m0, 
			     double min, double max, double peak_val, 
			     size_t num_bin );

#endif // __INC_MODEL_H__
