// *************************************************************************
//  oasys.cpp
// *************************************************************************

#include "oasys.h"

// *************************************************************************
// main function.
int main( int argc, char* argv[] ){
  // Get options
  cerr << endl;
  cerr << "Getting options...";
  options opt( argc, argv );
  opt.print();
  opt.print( opt.get_opt_file() );

  // Record start time
  time_t sttTime = time(0);

  // Load hit files
  cerr << "Loading input files..." << endl;
  time_t baseTime   = time(0);
  string orgname_in1 = opt.get_orgname_in1();
  string orgname_in2 = opt.get_orgname_in2();
  bool   flg_group   = opt.get_flg_group();
  gene_set gene_set_1( opt.get_genefile_1() );
  cerr << gene_set_1.size() << " genes in " << orgname_in1 << "." << endl;
  gene_set gene_set_2( opt.get_genefile_2() );
  cerr << gene_set_2.size() << " genes in " << orgname_in2 << "." << endl;
  if( flg_group ){ 
    group_set group_set_12;
    group_set_12.load( opt.get_group_file() );
    cerr << group_set_12.size() << " ortholog groups." << endl;
    cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
    cerr << endl;

    // Calculate seed orthologs
    cerr << "Calculating seed orthologs..." << endl;
    baseTime = time(0);
    ortholog_set seed_set_12 = group_set_12.extract_seed_set();
    cerr << seed_set_12.size() << " seed orthologs were found." << endl;
    cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
    cerr << endl;

    // Compute position matrix 
    cerr << "Computing position matrix..." << endl;
    baseTime = time(0);
    c_hit_set c_hit_set_12 = group_set_12.get_all_gene_pairs();
    position_map pos_map_1( gene_set_1 );
    cerr << pos_map_1.gene_size() << " genes in "
	 << opt.get_orgname_in1() << " were registered in our position map."
	 << endl;
    position_map pos_map_2( gene_set_2 );
    cerr << pos_map_2.gene_size() << " genes in "
	 << opt.get_orgname_in2() << " were registered in our position map."
	 << endl;
    position_2d_map pos_2d_map_12( pos_map_1, pos_map_2, c_hit_set_12 );
    cerr << pos_2d_map_12.hit_size() << " hits were registered "
	 << "in our 2D position map." << endl;
    cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
    cerr << endl;

    // Extract positional orthologs from many-to-many orthology assignments
    cerr << "Extracting positional orthologs from many-to-many "
	 << "orthology assignments..." << endl;
    baseTime = time(0);
    double col_index = opt.get_col_index();
    double sds       = opt.get_sds();
    double t_dpd     = opt.get_t_dpd();
    ortholog_set ortholog_set_12 
      = group_set_12.extract_ortholog_set( pos_2d_map_12, seed_set_12, 
					   col_index, sds, t_dpd );
    cerr << ortholog_set_12.size() << " orthologs were detected." << endl;
    ortholog_set_12.print_info( opt.get_ortholog_file() );
    group_set_12.print_info( opt.get_wnnso_file() );
    cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
    cerr << endl;
    return 0;
  }

  hit_set hit_set_12( opt.get_hitfile_12() );
  cerr << hit_set_12.size() << " hits between " 
       << orgname_in1 << " and " << orgname_in2 << "." << endl;
  hit_set hit_set_21( opt.get_hitfile_21() );
  cerr << hit_set_21.size() << " hits between " 
       << orgname_in2 << " and " << orgname_in1 << "." << endl;
  cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Calculate bi-directional hits
  cerr << "Calculating bi-directional hits..." << endl;
  baseTime = time(0);
  c_hit_set c_hit_set_12( hit_set_12, hit_set_21 );
  cerr << c_hit_set_12.size() << " bi-directional hits were found." << endl;
  hit_set_12.clear();
  hit_set_21.clear();
  cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Calculate seed orthologs
  cerr << "Calculating seed orthologs..." << endl;
  baseTime = time(0);
  string jobname  = opt.get_jobname();
  bool   flg_plot = opt.get_flg_plot();
  top_hit_set top_hit_set_12( c_hit_set_12, true );
  cerr << top_hit_set_12.size() << " top hits for " 
       << opt.get_orgname_in1() << endl;
  top_hit_set top_hit_set_21( c_hit_set_12, false );
  cerr << top_hit_set_21.size() << " top hits for "
       << opt.get_orgname_in2() << endl;
  ortholog_set rbh_set_12 = calc_rbh( top_hit_set_12, top_hit_set_21 );
  cerr << rbh_set_12.size() << " RBH pairs were detected." << endl;
  if( !flg_group )
    rbh_set_12.print_info( opt.get_bit_rbh_file() );
  ortholog_set seed_set_12 = rbh_set_12;
  cerr << seed_set_12.size() << " seed orthologs were found." << endl;
  ortholog_set non_seed_set_12
    = calc_non_seed_gene_pair( c_hit_set_12, seed_set_12 );
  cerr << non_seed_set_12.size() << " non-seed gene pairs were found." << endl;
  top_hit_set_12.clear();
  top_hit_set_21.clear();
  cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Compute position matrix 
  cerr << "Computing position matrix..." << endl;
  baseTime = time(0);
  position_map pos_map_1( gene_set_1 );
  cerr << pos_map_1.gene_size() << " genes in "
       << opt.get_orgname_in1() << " were registered in our position map."
       << endl;
  position_map pos_map_2( gene_set_2 );
  cerr << pos_map_2.gene_size() << " genes in "
       << opt.get_orgname_in2() << " were registered in our position map."
       << endl;
  position_2d_map pos_2d_map_12( pos_map_1, pos_map_2, c_hit_set_12 );
  cerr << pos_2d_map_12.hit_size() << " hits were registered "
       << "in our 2D position map." << endl;
  cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;


  // Count weighted number of neighbor seed orthologs (WNNSO)
  cerr << "Counting weighted number of neighbor seed orthologs (WNNSO)..." 
       << endl;
  baseTime = time(0);
  double col_index = opt.get_col_index();
  double sds       = opt.get_sds();
  double t_dpd     = opt.get_t_dpd();
  string oasys_hit_file = opt.get_oasys_hit_file();
  oasys_hit_set oasys_hit_set_12;
  if( opt.get_flg_short_cut() ){
    oasys_hit_set_12.set( oasys_hit_file );
  } else {
    oasys_hit_set_12.set( c_hit_set_12, pos_2d_map_12, seed_set_12, 
			  col_index, sds, t_dpd );
  }
  cerr << oasys_hit_set_12.size() << " WNNSO values were counted." << endl;
  ortholog_set wnnso_rbh_set_12 = oasys_hit_set_12.get_wnnso_rbh();
  wnnso_rbh_set_12.print_info( opt.get_wnnso_rbh_file() );
  c_hit_set_12.clear();
  cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Construct models
  cerr << "Constructing models..." << endl;
  baseTime = time(0);
  vector<double> w0, w1;
  map<string, oasys_hit>::iterator it = oasys_hit_set_12.begin();
  while( it != oasys_hit_set_12.end() ){
    string ip = it->first;
    w0.push_back( seed_set_12.exist( ip ) ? 0 : 1 );
    w1.push_back( seed_set_12.exist( ip ) ? 1 : 0 );
    ++it;
  }
  vector<double> wnnso_list  = oasys_hit_set_12.get_wnnso_list();
  vector<double> bit_list = oasys_hit_set_12.get_bit_list();
  cerr << " WNNSO for paralogs" << endl;
  model model_wnnso_p( wnnso_list, w0 );
  cerr << endl;
  cerr << " Bit score for paralogs" << endl;
  model model_bit_p( bit_list, w0 );
  cerr << endl;
  cerr << " WNNSO for main orthologs" << endl;
  model model_wnnso_mo( wnnso_list, w1 );
  cerr << endl;
  cerr << " Bit score for main orthologs" << endl;
  model model_bit_mo( bit_list, w1 );
  cerr << endl;
  cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;
  
  // Calculate scores
  cerr << "Calculating scores..." << endl;
  baseTime = time(0);
  double w_ratio = opt.get_w_ratio();
  oasys_hit_set_12.calc_syn_score( model_wnnso_mo, model_wnnso_p );
  oasys_hit_set_12.calc_mi_syn_score( model_wnnso_mo, model_wnnso_p );
  oasys_hit_set_12.calc_seq_score( model_bit_mo, model_bit_p );
  oasys_hit_set_12.calc_mi_seq_score( model_bit_mo, model_bit_p );
  oasys_hit_set_12.set_score( w_ratio );
  oasys_hit_set_12.set_mi_score( w_ratio );
  oasys_hit_set_12.print( oasys_hit_file );
  cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Detect main orthologs
  cerr << "Detecting main orthologs..." << endl;
  baseTime = time(0);
  ortholog_set oasys_rbh_set_12 = oasys_hit_set_12.get_oasys_rbh();
  oasys_rbh_set_12.print_info( opt.get_oasys_rbh_file() );
  cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  if( flg_plot ){
    /*
    // Draw histograms
    cerr << "Drawing histograms..." << endl;
    baseTime = time(0);
    draw_histogram( jobname+".seeds.wnnso", wnnso_list, w1 );
    draw_histogram( jobname+".non_seeds.wnnso", wnnso_list, w0 );
    draw_histogram( jobname+".seeds.bit", bit_list, w1 );
    draw_histogram( jobname+".non_seeds.bit", bit_list, w0 );
    cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
    cerr << endl;
    */

    // Draw PDFs
    cerr << "Drawing probability density functions..." << endl;
    baseTime = time(0);
    model_wnnso_mo.draw_pdf( jobname+".seeds.wnnso", wnnso_list, w1 );
    model_wnnso_p.draw_pdf( jobname+".non_seeds.wnnso", wnnso_list, w0 );
    model_bit_mo.draw_pdf( jobname+".seeds.bit", bit_list, w1 );
    model_bit_p.draw_pdf( jobname+".non_seeds.bit", bit_list, w0 );
    cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
    cerr << endl;

    // Draw score functions
    cerr << "Drawing score functions..." << endl;
    baseTime = time(0);
    oasys_hit_set_12.draw_score_functions( jobname, 
					   model_wnnso_mo, model_wnnso_p, 
					   model_bit_mo, model_bit_p );
    cerr << "done (" << time(0) - baseTime << " seconds)" << endl;
    cerr << endl;
  }

  // Print total time
  cerr << "Total time : " << time(0) - sttTime << " seconds" << endl;

  return 0;
}

