// *************************************************************************
//  oasys_hit.cpp
// *************************************************************************

#include "oasys_hit.h"
using namespace std;

// *************************************************************************
// Class oasys_hit
// Constructor
oasys_hit::oasys_hit( string str ){
  list<string> cols = split( str, "\t" );
  if( cols.size() != 4 )
    throw "Inappropriate file format: " + str;
  list<string>::iterator it = cols.begin();
  string id_pair = *it; ++it;
  double bit     = atof( it->c_str() ); ++it;
  double wnnso   = atof( it->c_str() ); ++it;
  set_IDpair( id_pair );
  set_bit( bit );
  set_wnnso( wnnso );
}


// *************************************************************************
// Class oasys_hit
// Constructor
oasys_hit::oasys_hit( c_hit ch, double wn ){
  set_IDpair( ch.get_IDpair() );
  set_bit( ch.get_bit() );
  set_wnnso( wn );
}


// *************************************************************************
// Class oasys_hit
// Constructor
oasys_hit::oasys_hit( string id_pair, double bit, double wn ){
  set_IDpair( id_pair );
  set_bit( bit );
  set_wnnso( wn );
}


// *************************************************************************
// Class oasys_hit
// Calculate and set the synteny score
void oasys_hit::calc_syn_score( model& m1, model& m0 ){
  syn_score = calc_score( m1, m0, wnnso );
}


// *************************************************************************
// Class oasys_hit
// Calculate and set the monotonically increasing synteny score
void oasys_hit::calc_mi_syn_score( model& m1, model& m0, double peak_val ){
  mi_syn_score = calc_mi_score( m1, m0, wnnso, peak_val );
}


// *************************************************************************
// Class oasys_hit
// Calculate and set the sequence score
void oasys_hit::calc_seq_score( model& m1, model& m0 ){
  seq_score = calc_score( m1, m0, get_bit() );
}


// *************************************************************************
// Class oasys_hit
// Calculate and set the monotonically increasing sequence score
void oasys_hit::calc_mi_seq_score( model& m1, model& m0, double peak_val ){
  mi_seq_score = calc_mi_score( m1, m0, get_bit(), peak_val );
}


// *************************************************************************
// Class oasys_hit
// Calculate and set the integrated score
void oasys_hit::set_score( double w_ratio ){
  score = w_ratio*syn_score + seq_score;
}


// *************************************************************************
// Class oasys_hit
// Calculate and set the integrated score (monotonically increasing version)
void oasys_hit::set_mi_score( double w_ratio ){
  mi_score = w_ratio*mi_syn_score + mi_seq_score;
}
