#ifndef __INC_OASYS_HIT_SET_H__
#define __INC_OASYS_HIT_SET_H__

#define HIST_BIN_OASYS_HIT_SET 100

#include <map>
#include <string>
#include <cmath>
#include "c_hit_set.h"
#include "ortholog_set.h"
#include "position_2d_map.h"
#include "oasys_hit.h"
#include "model.h"
#include "histogram.h"
#include "util.h"
#include "progress_bar.h"

#ifndef DEFAULT_NUM_BIN
#define DEFAULT_NUM_BIN 500
#endif // DEFAULT_NUM_BIN

class oasys_hit_set {
  map<string, oasys_hit> IDpair2hit;
  double max_wnnso, min_wnnso, ave_wnnso;

  void insert( string id_pair, oasys_hit o_hit )
  { IDpair2hit.insert( map<string, oasys_hit>
		       ::value_type( id_pair, o_hit ) ); }
  void set_stats();
 public:
  void set( c_hit_set& c_hs, position_2d_map& pos_2d_map, 
	    ortholog_set& seeds, double col_index, double sds, double t_dpd );
  void set( c_hit_set& c_hs, position_2d_map& pos_2d_map, 
	    ortholog_set& seeds, double col_index, double sds, double t_dpd, 
	    bool flg );
  void set( string info_file );
  double calc_wnnso( string id_pair, position_2d_map& pos_2d_map, 
		     ortholog_set& seeds, double col_index, 
		     double sds, double t_dpd );
  double dpd2weight( unsigned dpd, double sds );

  map<string, oasys_hit>::iterator begin() { return IDpair2hit.begin(); }
  map<string, oasys_hit>::iterator end() { return IDpair2hit.end(); }
  map<string, oasys_hit>::iterator find( string id_pair )
    { return IDpair2hit.find( id_pair ); }

  vector<double> get_wnnso_list();
  vector<double> get_wnnso_list( ortholog_set& os );
  vector<double> get_wnnso_list( ortholog_set& os1, ortholog_set& os2 );
  double get_wnnso_min();
  double get_wnnso_max();
  vector<double> get_bit_list();
  double get_bit_min();
  double get_bit_max();
  vector<double> get_syn_score_list();
  double get_syn_score_min();
  double get_syn_score_max();
  double get_syn_score_peak();
  vector<double> get_mi_syn_score_list();
  double get_mi_syn_score_min();
  double get_mi_syn_score_max();
  vector<double> get_seq_score_list();
  double get_seq_score_min();
  double get_seq_score_max();
  double get_seq_score_peak();
  vector<double> get_mi_seq_score_list();
  double get_mi_seq_score_min();
  double get_mi_seq_score_max();
  vector<double> get_score_list();
  double get_score_min();
  double get_score_max();
  vector<double> get_mi_score_list();
  double get_mi_score_min();
  double get_mi_score_max();
  hit_set get_wnnso_hit_set( bool flg );
  hit_set get_mi_score_hit_set( bool flg );
  ortholog_set get_wnnso_rbh();
  ortholog_set get_oasys_rbh();

  void calc_syn_score( model& m1, model& m0 );
  void calc_mi_syn_score( model& m1, model& m0 );
  void calc_seq_score( model& m1, model& m0 );
  void calc_mi_seq_score( model& m1, model& m0 );
  void set_score( double w_ratio );
  void set_mi_score( double w_ratio );

  void print( string outfile );
  void print_bit( string outfile );
  void print_bit( ortholog_set& os, string outfile );
  void print_wnnso( string outfile );
  void print_wnnso( ortholog_set& os, string outfile );
  void print_i_score( gene_set& gs1, gene_set& gs2, 
		      string outfile_1, string outfile_2 );

  void draw_syn_score_histogram( string name );
  void draw_mi_syn_score_histogram( string name );
  void draw_seq_score_histogram( string name );
  void draw_mi_seq_score_histogram( string name );
  void draw_score_histogram( string name );
  void draw_mi_score_histogram( string name );
  void draw_score_functions( string name, model& w1, model& w0, 
			     model& b1, model& b0 );

  unsigned size() { return IDpair2hit.size(); }
  void clear() { IDpair2hit.clear(); }
};

#endif // __INC_OASYS_HIT_SET_H__
