#ifndef __INC_OGG_H__
#define __INC_OGG_H__

#define STEP_SIZE_OGG 0.01
#define TOL_OGG 1e-4
#define EPSABS_OGG 1e-3
#define ITER_OGG 100
#define NEARLY_ZERO_OGG 1e-100
#define PLOT_SAMPLES_OGG 10000
#define NUM_HIST_BIN_OGG 50
#define MIN_SD_OGG 0.01
#define MIN_SHAPE_OGG 0.1
#define M_OGG 3

#include <cmath>
#include <iostream>
#include <fstream>
#include <gsl/gsl_sf_psi.h>
#include <gsl/gsl_sf_gamma.h>
#include <gsl/gsl_multimin.h>
#include "param.h"
#include "pdf.h"
#include "histogram.h"
#include "util.h"

using namespace std;

class pdf_ogg;
class param_ogg : public param {
  friend class pdf_ogg;
  double loc, sd, shape;
 public:
  virtual ~param_ogg(){}
  param_ogg(){};
  param_ogg( double d1, double d2, double d3 ){ loc=d1; sd=d2; shape=d3; }
  param_ogg( vector<double>& x );
  param_ogg( vector<double>& x, vector<double>& w );
  void set_shape( double c ){ shape = c; }
  void show();
};


class pdf_ogg : public pdf {
  param_ogg* p;
 public:
  virtual ~pdf_ogg(){}
  virtual void set_param( void* par ){ p = (param_ogg*)par; }
  virtual void fit_ml( vector<double>& x );
  virtual void fit_ml( vector<double>& x, vector<double>& w );
  virtual double eval( double x );
  virtual double log_L( vector<double>& x );
  virtual double log_L( vector<double>& x, vector<double>& w );
  virtual double log_dL( vector<double>& x );
  virtual double log_dL( vector<double>& x, vector<double>& w );
  virtual double aic( vector<double>& x ){ return log_L(x)-M_OGG; }
  virtual double aic( vector<double>& x, vector<double>& w )
  { return log_L(x, w)-M_OGG; }
  virtual void print_sample_points( string file, double min, double max );
  virtual void plot( string name );
  virtual void plot( string name, vector<double>& x, size_t num_bin );
  virtual void plot( string name, vector<double>& x, vector<double>& w, 
		     size_t num_bin );
  virtual void plot( string name, histogram& h );
  virtual void plot( string name, vector<void*>& param_list );
  double cal_minimum( vector<double>& x );
  double cal_sd( vector<double>& x );
  double cal_sd( vector<double>& x, vector<double>& w );
  double cal_shape( vector<double>& x );
  double cal_shape( vector<double>& x, vector<double>& w );
};


typedef pair< vector<double>*, param_ogg* > gsl_param_ogg;
double ogg_f( const gsl_vector* v, void* gsl_param );
void ogg_df( const gsl_vector* v, void* gsl_param, gsl_vector* df );
void ogg_fdf( const gsl_vector* v, void* gsl_param, 
	      double* f, gsl_vector* df );
typedef vector<void*> gsl_param_w_ogg;
double w_ogg_f( const gsl_vector* v, void* gsl_param );
void w_ogg_df( const gsl_vector* v, void* gsl_param, gsl_vector* df );
void w_ogg_fdf( const gsl_vector* v, void* gsl_param, 
		double* f, gsl_vector* df );

#endif // __INC_OGG_H__
