#ifndef __INC_ORTHOLOG_SET_H__
#define __INC_ORTHOLOG_SET_H__

#include <map>
#include <string>
#include "gene_set.h"
#include "c_hit_set.h"
#include "top_hit_set.h"
#include "util.h"

#ifndef DEFAULT_WIDTH
#define DEFAULT_WIDTH 10.0
#endif // DEFAULT_WIDTH

class ortholog_set {
  map<string, double> gp2score;

 public:
  ortholog_set(){}
  ortholog_set( string in_file );
  void set_fs_rate( top_hit_set& t_hs, bool flg );

  void insert( string ip, double score );
  void insert( string g1, string g2, double score ) 
  { insert( g1+' '+g2, score ); }
  void erase( string g1, string g2 ){ erase( g1+' '+g2 ); }
  void erase( string ip ) { gp2score.erase( ip ); }

  map<string, double>::iterator begin() { return gp2score.begin(); }
  map<string, double>::iterator end() { return gp2score.end(); }
  map<string, double>::iterator find( string ip ) 
  { return gp2score.find( ip ); }
  double get_score( string ip );
  c_hit_set get_c_hit_set();

  void print_IDpair( string outfile );
  void print_info(  string outfile );
  void print_anchor( gene_set& gs1, gene_set& gs2, string outfile );

  bool exist( string ip ) { return find(ip)==end() ? false : true; }
  unsigned size() { return (size_t)gp2score.size(); }
  void clear() { gp2score.clear(); }
};

ortholog_set calc_non_seed_gene_pair( c_hit_set& c_hs, ortholog_set& ss );
ortholog_set calc_rbh( top_hit_set& t_hs_12, top_hit_set& t_hs_21 );

#endif // __INC_ORTHOLOG_SET_H__
