// *************************************************************************
//  position_2d_map.cpp
// *************************************************************************

#include "position_2d_map.h"
using namespace std;


// *************************************************************************
// Class position_2d_map
// Constructor
position_2d_map::position_2d_map( position_map& pm_1, 
				  position_map& pm_2, 
				  c_hit_set& c_hs ){
  map<string, c_hit_set> cp2hs = get_chrom_pair2hit_set( c_hs, pm_1, pm_2 );
  map<string, c_hit_set>::iterator it = cp2hs.begin();
  while( it != cp2hs.end() ){
    string chr_pair  = it->first;
    c_hit_set tmp_hs = it->second;
    list<string> chrs = split( chr_pair, " " );
    list<string>::iterator chr_it = chrs.begin();
    string chr_1 = *chr_it; ++chr_it;
    string chr_2 = *chr_it; ++chr_it;
    position_list pl_1 = pm_1.get_position_list( chr_1 );
    position_list pl_2 = pm_2.get_position_list( chr_2 );
    position_matrix p_mat( pl_1, pl_2, tmp_hs );
    insert( chr_pair, p_mat );
    ++it;
  }
  set_idpair2chrpair( pm_1, pm_2, c_hs );
  set_idpair2sign( pm_1, pm_2, c_hs );
}


// *************************************************************************
// Class position_2d_map
// Set a variable idpair2chrpair
void position_2d_map::set_idpair2chrpair( position_map& pm_1, 
					  position_map& pm_2, 
					  c_hit_set& c_hs ){
  map<string, c_hit>::iterator hs_it = c_hs.begin();
  while( hs_it != c_hs.end() ){
    string id_pair = hs_it->first;
    list<string> IDs = split( id_pair, " " );
    list<string>::iterator id_it = IDs.begin();
    string id_1 = *id_it; ++id_it;
    string id_2 = *id_it; ++id_it;
    string chr_1 = pm_1.get_chrom( id_1 );
    string chr_2 = pm_2.get_chrom( id_2 );
    string chr_pair = chr_1 + " " + chr_2;
    ip_insert( id_pair, chr_pair );
    ++hs_it;
  }
}


// *************************************************************************
// Class position_2d_map
// Set a variable idpair2sign
void position_2d_map::set_idpair2sign( position_map& pm_1, 
				       position_map& pm_2, 
				       c_hit_set& c_hs ){
  map<string, c_hit>::iterator hs_it = c_hs.begin();
  while( hs_it != c_hs.end() ){
    string id_pair = hs_it->first;
    list<string> IDs = split( id_pair, " " );
    list<string>::iterator id_it = IDs.begin();
    string id_1 = *id_it; ++id_it;
    string id_2 = *id_it; ++id_it;
    bool   sgn_1 = pm_1.get_sign( id_1 );
    bool   sgn_2 = pm_2.get_sign( id_2 );
    bool   sgn   = (sgn_1 == sgn_2) ? true : false;
    ip2sgn_insert( id_pair, sgn );
    ++hs_it;
  }
}


// *************************************************************************
// Class position_2d_map
// Return a set of pairs of chromosomes included in the map
map<string, bool> position_2d_map::get_chrom_pairs( position_map& pm1, 
						    position_map& pm2 ){
  map<string, bool> chrom_pairs;
  map<string, bool> chroms_1 = pm1.get_chroms();
  map<string, bool>::iterator it_1 = chroms_1.begin();
  map<string, bool> chroms_2 = pm2.get_chroms();
  map<string, bool>::iterator it_2 = chroms_2.begin();
  while( it_1 != chroms_1.end() ){
    string chr_1 = it_1->first;
    while( it_2 != chroms_2.end() ){
      string chr_2 = it_2->first;
      string chrom_pair = chr_1 + " " + chr_2;
      chrom_pairs.insert( map<string, bool>::value_type( chrom_pair, true ) );
      ++it_2;
    }
    ++it_1;
  }
  return chrom_pairs;
}


// *************************************************************************
// Class position_2d_map
// Return a set of hits detected between a given pair of chromosomes
c_hit_set position_2d_map::get_hit_set( c_hit_set& c_hs, 
					position_map& pm_1, 
					position_map& pm_2, 
					string chr_pair ){
  c_hit_set tmp_hs;
  map<string, c_hit>::iterator hs_it = c_hs.begin();
  while( hs_it != c_hs.end() ){
    string id_pair   = hs_it->first;
    c_hit  hit       = hs_it->second;
    list<string> IDs = split(id_pair, " ");
    list<string>::iterator id_it = IDs.begin();
    string id_1 = *id_it; ++id_it;
    string id_2 = *id_it; ++id_it;
    string cur_chr_1 = pm_1.get_chrom( id_1 );
    string cur_chr_2 = pm_2.get_chrom( id_2 );
    string cur_chr_pair = cur_chr_1 + " " + cur_chr_2;
    if( cur_chr_pair == chr_pair ){
      tmp_hs.insert( id_pair, hit );
    }
    ++hs_it;
  }
  return tmp_hs;
}


// *************************************************************************
// Class position_2d_map
// Return a set of hits detected between a given pair of chromosomes
map<string, c_hit_set> 
position_2d_map::get_chrom_pair2hit_set( c_hit_set& c_hs, 
					 position_map& pm_1, 
					 position_map& pm_2 ){
  map<string, c_hit_set> cp2hs;
  map<string, c_hit>::iterator hs_it = c_hs.begin();
  while( hs_it != c_hs.end() ){
    string id_pair   = hs_it->first;
    c_hit  hit       = hs_it->second;
    list<string> IDs = split(id_pair, " ");
    list<string>::iterator id_it = IDs.begin();
    string id_1 = *id_it; ++id_it;
    string id_2 = *id_it; ++id_it;
    string chr_1 = pm_1.get_chrom( id_1 );
    string chr_2 = pm_2.get_chrom( id_2 );
    string chr_pair = chr_1 + " " + chr_2;
    map<string, c_hit_set>::iterator cp2hs_it = cp2hs.find( chr_pair );
    if( cp2hs_it == cp2hs.end() ){
      c_hit_set tmp_hs;
      tmp_hs.insert( id_pair, hit );
      cp2hs.insert( map<string, c_hit_set>::value_type( chr_pair, tmp_hs ) );
    } else {
      cp2hs_it->second.insert( id_pair, hit );
    }
    ++hs_it;
  }
  return cp2hs;
}


// *************************************************************************
// Class position_2d_map
// Return a number of hits registered in this instance
unsigned position_2d_map::hit_size() {
  unsigned hit_size = 0;
  map<string, position_matrix>::iterator it = begin();
  while( it != end() ){
    hit_size += it->second.size();
    ++it;
  }
  return hit_size;
}


// *************************************************************************
// Class position_2d_map
// Return a pair of chromosome ID
string position_2d_map::get_chrom_pair( string ip ){
  map<string, string>::iterator it = ip_find( ip );
  if( it == ip_end() ){
    cerr << "Error: " << ip << " is not found."  << endl;
    exit( -1 );
  }
  return it->second;
}


// *************************************************************************
// Class position_2d_map
// Return a relative sign of a given gene pair
bool position_2d_map::get_sign( string ip ){
  map<string, bool>::iterator it = ip2sgn_find( ip );
  if( it == ip2sgn_end() ){
    cerr << "Error: " << ip << " is not found."  << endl;
    exit( -1 );
  }
  return it->second;
}


// *************************************************************************
// Class position_2d_map
// Return a diagonal pseudo distance between two gene pairs
unsigned position_2d_map::DPD( string id_pair_i, string id_pair_j, 
			       double col_index ){
  string chr_pair_i = get_chrom_pair( id_pair_i );
  string chr_pair_j = get_chrom_pair( id_pair_j );
  if( chr_pair_i == chr_pair_j ){
    position_matrix* p_mat = get_position_matrix( chr_pair_i );
    bool sgn_i = get_sign( id_pair_i );
    bool sgn_j = get_sign( id_pair_j );
    if( sgn_i == sgn_j ){
      return p_mat->DPD( id_pair_i, id_pair_j, sgn_i, col_index );
    } else {
      return INF;
    }
  } else {
    return INF;
  }
}


// *************************************************************************
// Class position_2d_map
// Return a position matrix specified by a pair of chromosome ID
position_matrix* position_2d_map::get_position_matrix( string chr_pair ){
  map<string, position_matrix>::iterator it = find( chr_pair );
  if( it == end() ){
    cerr << "Error: " << chr_pair << " is not found." << endl;
    exit( -1 );
  }
  return &(it->second);
}


