#ifndef __INC_POSITION_2D_MAP_H__
#define __INC_POSITION_2D_MAP_H__

#include <map>
#include "c_hit_set.h"
#include "position_map.h"
#include "position_matrix.h"

#ifndef INF
#define INF 0
#endif // INF

class position_2d_map {
  map<string, position_matrix> chrpair2pm;
  map<string, string> idpair2chrpair;
  map<string, bool> idpair2sign;

  void insert( string chr_pair, position_matrix pm )
  { chrpair2pm.insert( map<string, position_matrix>
		       ::value_type( chr_pair, pm )); }
  void ip_insert( string ip, string cp )
  { idpair2chrpair.insert( map<string, string>::value_type( ip, cp ) ); }
  void ip2sgn_insert( string ip, bool sgn )
  { idpair2sign.insert( map<string, bool>::value_type( ip, sgn ) ); }
 public:
  position_2d_map( position_map& pm_1, position_map& pm_2, c_hit_set& c_hs );
  void set_idpair2chrpair( position_map& pm_1, position_map& pm_2, 
			   c_hit_set& c_hs);
  void set_idpair2sign( position_map& pm_1, position_map& pm_2, 
			c_hit_set& c_hs);
  map<string, position_matrix>::iterator begin() 
    { return chrpair2pm.begin(); }
  map<string, position_matrix>::iterator end()
    { return chrpair2pm.end(); }
  map<string, position_matrix>::iterator find( string chr_pair )
    { return chrpair2pm.find( chr_pair ); }
  map<string, string>::iterator ip_begin() { return idpair2chrpair.begin(); }
  map<string, string>::iterator ip_end() { return idpair2chrpair.end(); }
  map<string, string>::iterator ip_find( string ip )
    { return idpair2chrpair.find( ip ); }
  map<string, bool>::iterator ip2sgn_begin() { return idpair2sign.begin(); }
  map<string, bool>::iterator ip2sgn_end() { return idpair2sign.end(); }
  map<string, bool>::iterator ip2sgn_find( string ip )
    { return idpair2sign.find( ip ); }
  map<string, bool> get_chrom_pairs( position_map& pm1, position_map& pm2 );
  string get_chrom_pair( string ip );
  bool get_sign( string ip );
  c_hit_set get_hit_set( c_hit_set& c_hs, position_map& pm_1, 
			 position_map& pm_2, string chr_pair );
  map<string, c_hit_set> get_chrom_pair2hit_set( c_hit_set& c_hs, 
						 position_map& pm_1, 
						 position_map& pm_2 );
  unsigned hit_size();
  unsigned DPD( string ip_i, string ip_j, double col_index );
  position_matrix* get_position_matrix( string chr_pair );
};

#endif // __INC_POSITION_2D_MAP_H__
