// *************************************************************************
//  position_list.cpp
// *************************************************************************

#include "position_list.h"
using namespace std;


// *************************************************************************
// Class position_list
// Constructor
position_list::position_list( gene_set& gs ){
  map<double, string> coords2id = calc_coords2id( gs );
  map<double, string>::iterator co_it = coords2id.begin();
  while( co_it != coords2id.end() ){
    string id = co_it->second;
    pos2id.push_back( id );
    unsigned pos = pos2id.size() - 1;
    id2pos_insert( id, pos );
    ++co_it;
  }
}


// *************************************************************************
// Class position_list
// Return the position of given gene 
unsigned position_list::get_pos( string id ){
  map<string, unsigned>::iterator it = id2pos_find( id );
  if( it == id2pos_end() ){
    cerr << "Error: " << id << " is not found." << endl;
  }
  return it->second;
}

// *************************************************************************
// Class position_list
// Compute a map from coordinate position to gene ID
map<double, string> position_list::calc_coords2id( gene_set& gs ){
  map<double, string> coords2id;
  map<string, gene>::iterator gs_it = gs.begin();
  while( gs_it != gs.end() ){
    string id = gs_it->first;
    gene   g  = gs_it->second;
    double coords = ( g.get_stt() + g.get_end() ) / 2.0;
    map<double, string>::iterator co_it = coords2id.find( coords );
    while( co_it != coords2id.end() ){
      coords += DELTA;
      co_it   = coords2id.find( coords );
    }
    coords2id.insert( map<double, string>::value_type( coords, id ) );
    ++gs_it;
  }
  return coords2id;
}


