#ifndef __INC_POSITION_MAP_H__
#define __INC_POSITION_MAP_H__

#include <vector>
#include <map>
#include "gene_set.h"
#include "position_list.h"

using namespace std;

class position_map {
  map<string, position_list> chr2pl;
  map<string, string> id2chr;
  map<string, bool> id2sgn;

  void insert( string chr, position_list pl )
  { chr2pl.insert( map<string, position_list>::value_type( chr, pl ) ); }
  void id_insert( string id, string chr )
  { id2chr.insert( map<string, string>::value_type( id, chr ) ); }
  void id_insert( string id, bool flg )
  { id2sgn.insert( map<string, bool>::value_type( id, flg ) ); }
  void set_id2chr( gene_set& gs );
  void set_id2sgn( gene_set& gs );
  gene_set get_gene_set( gene_set& gs, string chr );
 public:
  position_map( gene_set& gs );
  map<string, position_list>::iterator begin() { return chr2pl.begin(); }
  map<string, position_list>::iterator end() { return chr2pl.end(); }
  map<string, position_list>::iterator find( string chr ) 
    { return chr2pl.find( chr ); }
  map<string, string>::iterator id_begin() { return id2chr.begin(); }
  map<string, string>::iterator id_end() { return id2chr.end(); }
  map<string, string>::iterator id_find( string id ) 
    { return id2chr.find( id ); }
  map<string, bool>::iterator id2sgn_begin() { return id2sgn.begin(); }
  map<string, bool>::iterator id2sgn_end() { return id2sgn.end(); }
  map<string, bool>::iterator id2sgn_find( string id )
    { return id2sgn.find( id ); }
  position_list get_position_list( string chr );
  map<string, bool> get_chroms();
  map<string, bool> get_chroms( gene_set& gs );
  string get_chrom( string id );
  bool get_sign( string id );
  unsigned gene_size();
};

#endif // __INC_POSITION_MAP_H__
