// *************************************************************************
//  position_matrix.cpp
// *************************************************************************

#include "position_matrix.h"
using namespace std;

// *************************************************************************
// Class position_matrix
// Constructor
position_matrix::position_matrix( unsigned num_cells ){
  unsigned base_i = num_cells;
  unsigned base_j = num_cells;
  for( unsigned i=0 ; i<=base_i+num_cells ; ++i ){
    for( unsigned j=0 ; j<=base_i+num_cells ; ++j ){
      int coord_i = (int)i - (int)base_i;
      int coord_j = (int)j - (int)base_j;
      string id_pair = int2string(coord_i) + " " + int2string(coord_j);
      mat_idx mat_idx = mat_idx::pair( i, j );
      insert( mat_idx, id_pair );
    }
  }
}


// *************************************************************************
// Class position_matrix
// Constructor
position_matrix::position_matrix( unsigned n_cols, unsigned n_rows ){
  for( unsigned i=1 ; i<=n_cols ; ++i ){
    for( unsigned j=1 ; j<=n_rows ; ++j ){
      string id_pair = int2string(i) + " " + int2string(j);
      mat_idx mat_idx = mat_idx::pair( i, j );
      insert( mat_idx, id_pair );
    }
  }
}


// *************************************************************************
// Class position_matrix
// Constructor
position_matrix::position_matrix( position_list& pl_1, 
				  position_list& pl_2, 
				  c_hit_set& c_hs ){
  map<string, c_hit>::iterator hs_it = c_hs.begin();
  while( hs_it != c_hs.end() ){
    string ip = hs_it->first;
    list<string> IDs = split(ip, " ");
    list<string>::iterator id_it = IDs.begin();
    string id_1 = *id_it; ++id_it;
    string id_2 = *id_it; ++id_it;
    unsigned pos_1 = pl_1.get_pos( id_1 );
    unsigned pos_2 = pl_2.get_pos( id_2 );
    mat_idx  idx   = mat_idx::pair( pos_1, pos_2 );
    insert( idx, ip );
    ++hs_it;
  }
}


// *************************************************************************
// Class position_matrix
// Return a diagonal pseudo distance specified by two gene ID pairs
unsigned position_matrix::DPD( string id_pair_i, string id_pair_j, bool sgn, 
			       double col_index ){
  mat_idx mi_i = get_mat_idx( id_pair_i );
  mat_idx mi_j = get_mat_idx( id_pair_j );
  if( is_collinear( mi_i, mi_j, sgn ) ){
    return DPD( mi_i, mi_j, col_index );
  } else {
    return INF;
  }
}


// *************************************************************************
// Class position_matrix
// Return a diagonal pseudo distance specified by two matrix indices
unsigned position_matrix::DPD( mat_idx mi_i, mat_idx mi_j, bool sgn, 
			       double col_index ){
  if( is_collinear( mi_i, mi_j, sgn ) ){
    return DPD( mi_i, mi_j, col_index );
  } else {
    return INF;
  }
}


// *************************************************************************
// Class position_matrix
// Return a diagonal pseudo distance specified by two matrix indices
unsigned position_matrix::DPD( mat_idx mi_i, mat_idx mi_j, double col_index ){
  unsigned pos_i_1 = mi_i.first;
  unsigned pos_i_2 = mi_i.second;
  unsigned pos_j_1 = mi_j.first;
  unsigned pos_j_2 = mi_j.second;
  unsigned DPD = (col_index+1) 
    * util::max( util::abs( (int)pos_i_1 - (int)pos_j_1 ), 
		 util::abs( (int)pos_i_2 - (int)pos_j_2 ) )
    - col_index
    * util::min( util::abs( (int)pos_i_1 - (int)pos_j_1 ), 
		 util::abs( (int)pos_i_2 - (int)pos_j_2 ) );

//   cerr << pos_i_1 << " " << pos_i_2 << " "
//        << pos_j_1 << " " << pos_j_2 << " "
//        << sgn << endl
//        << util::max( util::abs( (int)pos_i_1 - (int)pos_j_1 ), 
// 		     util::abs( (int)pos_i_2 - (int)pos_j_2 ))
//        << " "
//        << util::min( util::abs( (int)pos_i_1 - (int)pos_j_1), 
// 		     util::abs( (int)pos_i_2 - (int)pos_j_2 )) 
//        << " "
//        << DPD << endl << endl; 
  
  return DPD;
}


// *************************************************************************
// Class position_matrix
// Return whether given two gene pairs are collinear, or not
bool position_matrix::is_collinear( mat_idx mi_i, mat_idx mi_j, bool sgn ){
  unsigned pos_i_1 = mi_i.first;
  unsigned pos_i_2 = mi_i.second;
  unsigned pos_j_1 = mi_j.first;
  unsigned pos_j_2 = mi_j.second;
  if( pos_i_1 == pos_j_1 ){ return false; }
  if( pos_i_2 == pos_j_2 ){ return false; }
  double slope = (double)( (int)pos_i_2 - (int)pos_j_2 ) 
    / ( (int)pos_i_1 - (int)pos_j_1 );
  if( sgn ){ return slope>0 ? true : false; }
  else { return slope<0 ? true : false; }
}


// *************************************************************************
// Class position_matrix
// Return an indices of matrix
mat_idx position_matrix::get_mat_idx( string id_pair ){
  map< string, mat_idx >::iterator it = id_find( id_pair );
  if( it == id_end() ){
    cerr << "Error!: " << id_pair << " is not found." << endl;
    exit( -1 );
  }
  return it->second;
}
