#ifndef __INC_POSITION_MATRIX_H__
#define __INC_POSITION_MATRIX_H__

#include <vector>
#include <map>
#include "c_hit_set.h"
#include "position_list.h"

#ifndef INF
#define INF 0
#endif // INF

using namespace std;
typedef pair<unsigned, unsigned> mat_idx;

class position_matrix {
  map< mat_idx, string > idx2id;
  map< string, mat_idx > id2idx;

  void insert( mat_idx idx, string id )
  { idx2id.insert( map< mat_idx, string >::value_type( idx, id ) );
    id2idx.insert( map< string, mat_idx >::value_type( id, idx ) ); }
 public:
  position_matrix( unsigned num_cells );
  position_matrix( unsigned n_cols, unsigned n_rows );
  position_matrix( position_list& pl_1, position_list& pl_2, c_hit_set& c_hs );
  map< mat_idx, string >::iterator idx_begin() { return idx2id.begin(); }
  map< mat_idx, string >::iterator idx_end() { return idx2id.end(); }
  map< mat_idx, string >::iterator idx_find( mat_idx mi )
    { return idx2id.find( mi ); }
  map< string, mat_idx >::iterator id_begin() { return id2idx.begin(); }
  map< string, mat_idx >::iterator id_end() { return id2idx.end(); }
  map< string, mat_idx >::iterator id_find( string id )
    { return id2idx.find( id ); }
  unsigned size() { return idx2id.size(); }
  unsigned DPD( string id_pair_i, string id_pair_j, bool sgn, 
		double col_index );
  unsigned DPD( mat_idx mi_i, mat_idx mi_j, bool sgn, double col_index );
  unsigned DPD( mat_idx mi_i, mat_idx mi_j, double col_index );
  bool is_collinear( mat_idx mi_i, mat_idx mi_j, bool sgn );
  mat_idx get_mat_idx( string id_pair );
};

#endif // __INC_POSITION_MATRIX_H__
