#ifndef __INC_SEED_MATRIX_H__
#define __INC_SEED_MATRIX_H__

#include <cmath>
#include <iostream>
#include <string>
#include <list>
#include <map>
#include "util.h"

#ifndef INF
#define INF 0
#endif // INF

using namespace std;
typedef pair<unsigned, unsigned> coords;

class seed_matrix {
  unsigned n_cols;
  unsigned n_rows;
  map< coords, int > coords2sign;
  map< coords, double > coords2p_wnnso;
  map< coords, double > coords2r_wnnso;

  void initialize();
  void insert( coords c, int i )
  { coords2sign.insert( map< coords, int >::value_type( c, i ) ); }
  void erase( coords c ) { coords2sign.erase( c ); }
  void p_insert( coords c, double d )
  { coords2p_wnnso.insert( map< coords, double >::value_type( c, d ) ); }
  void p_erase( coords c ) { coords2p_wnnso.erase( c ); }
  void r_insert( coords c, double d )
  { coords2r_wnnso.insert( map< coords, double >::value_type( c, d ) ); }
  void r_erase( coords c ) { coords2r_wnnso.erase( c ); }
 public:
  seed_matrix( string in_file );
  seed_matrix( unsigned n_cols, unsigned n_rows, string seed_pos );
  void insert_seeds( string seed_pos );
  void calc_wnnso( double sigma );
  void calc_wnnso( coords c, double sigma );
  double calc_weight( coords c1, coords c2, bool sgn, double sigma );
  unsigned calc_dpd( coords c1, coords c2, bool sgn );
  bool is_collinear( coords c1, coords c2, bool sgn );
  double dpd2weight( unsigned dpd, double sigma );

  map< coords, int >::iterator begin() { return coords2sign.begin(); }
  map< coords, int >::iterator end() { return coords2sign.end(); }
  map< coords, int >::iterator find( coords c ) { return coords2sign.find(c); }
  map< coords, double >::iterator p_begin() { return coords2p_wnnso.begin(); }
  map< coords, double >::iterator p_end() { return coords2p_wnnso.end(); }
  map< coords, double >::iterator p_find( coords c )
  { return coords2p_wnnso.find( c ); }
  map< coords, double >::iterator r_begin() { return coords2r_wnnso.begin(); }
  map< coords, double >::iterator r_end() { return coords2r_wnnso.end(); }
  map< coords, double >::iterator r_find( coords c )
  { return coords2r_wnnso.find( c ); }

  unsigned get_n_cols() { return n_cols; }
  unsigned get_n_rows() { return n_rows; }
  unsigned num_seeds();

  void show();
  void print( string outfile );
  void p_print( string outfile );
  void r_print( string outfile );
  void draw( string name, string Rdir ) 
  { p_draw( name+".p", Rdir ); r_draw( name+".r", Rdir ); };
  void p_draw( string name, string Rdir );
  void r_draw( string name, string Rdir );

  void clear() { coords2sign.clear(); }
};

#endif // __INC_SEED_MATRIX_H__
