// *************************************************************************
//  top_hit_set.cpp
// *************************************************************************

#include "top_hit_set.h"
using namespace std;

// *************************************************************************
// Class top_hit_set
// Constructor
top_hit_set::top_hit_set( c_hit_set& c_hs, bool b ){
  map<string, c_hit>::iterator it = c_hs.begin();
  while( it != c_hs.end() ){
    string IDpair = it->first;
    list<string> IDs = split(IDpair, " ");
    list<string>::iterator id_it = IDs.begin();
    string ID_1, ID_2;
    if( b ){ ID_1 = *id_it; ++id_it; ID_2 = *id_it; ++id_it; }
    else   { ID_2 = *id_it; ++id_it; ID_1 = *id_it; ++id_it; }
    double bit  = it->second.get_bit();    
    bool flg_best = false;
    if( exist( ID_1 ) ){
      double cur_bit = find_bit( ID_1 )->second;
      double sec_bit = find_fs( ID_1 )->second;
      if( bit > sec_bit ){
	erase_fs( ID_1 );
	insert_fs( ID_1, bit );
      }
      if( bit > cur_bit ){
	erase_id( ID_1 ); 
	erase_bit( ID_1 ); 
	erase_fs( ID_1 );
	insert_fs( ID_1, cur_bit );
	flg_best = true;
      }
    } else {
      flg_best = true;
    }
    if( flg_best ){
      insert_id( ID_1, ID_2 );
      insert_bit( ID_1, bit );
      if( find_fs( ID_1 ) == end_fs() ){
	insert_fs( ID_1, 0.0 );
      }
    }
    ++it;
  }
  map<string, string>::iterator id_it = begin_id();
  while( id_it != end_id() ){
    string ID_1 = id_it->first;
    string ID_2 = id_it->second;
    rev_insert( ID_2, true );
    double best_bit = find_bit( ID_1 )->second;
    double sec_bit  = find_fs( ID_1 )->second;
    double fs_rate  = sec_bit / best_bit;
    erase_fs( ID_1 );
    insert_fs( ID_1, fs_rate );
    ++id_it;
  }
}


// *************************************************************************
// Class top_hit_set
// print bit scores
void top_hit_set::print_bit( string outfile ){
  ofstream ofs( outfile.c_str() );
  map<string, double>::iterator it = begin_bit();
  while( it != end_bit() ){
    double bit = it->second;
    ofs << bit << endl;
    ++it;
  }
}


// *************************************************************************
// Class top_hit_set
// print fs rates
void top_hit_set::print_fs( string outfile ){
  ofstream ofs( outfile.c_str() );
  map<string, double>::iterator it = begin_fs();
  while( it != end_fs() ){
    double fs = it->second;
    ofs << fs << endl;
    ++it;
  }
}


// *************************************************************************
// Calculate the number of tops
map<string, unsigned int> top_count( top_hit_set& t_hs ){
  map<string, unsigned int> id2count;
  map<string, string>::iterator id_it = t_hs.begin_id();
  while( id_it != t_hs.end_id() ){
    string id = id_it->second;
    unsigned int cnt = top_count( id2count, id ) + 1;
    id2count.erase( id );
    id2count.insert( map<string, unsigned int>::value_type( id, cnt ) );
    ++id_it;
  }
  return id2count;
}


// *************************************************************************
// Return the number of tops
unsigned int top_count( map<string, unsigned int>& id2cnt, string id ){
  map<string, unsigned int>::iterator it = id2cnt.find( id );
  unsigned int cnt = 0;
  if( it == id2cnt.end() ){ cnt = 0; }
  else { cnt = it->second; }
  return cnt;
}
