// *************************************************************************
//  util.cpp
// *************************************************************************

#include "util.h"
using namespace std;


// *************************************************************************
// Split string 
list<string> split(string str, string delim) {
  list<string> list;
  unsigned cut_pos;
  while( (cut_pos = str.find_first_of(delim)) != (unsigned)-1 ){
    if( cut_pos > 0 ){
      list.push_back(str.substr(0, cut_pos));
    }
    str = str.substr(cut_pos + 1);
  }
  if( str.length()>0 ){
    list.push_back(str);
  }
  return list;
}


// *************************************************************************
// Convert bool into string
string bool2string( bool b ){
  if(b) return (string)"yes";
  else  return (string)"no";
}


// *************************************************************************
// Convert int into string
string int2string( int i ){
  char c_str[STR_MAX];
  sprintf( c_str, "%d", i );
  return (string)c_str;
}


// *************************************************************************
// Convert double into string
string double2string( double d ){
  char c_str[STR_MAX];
  sprintf( c_str, "%lf", d );
  return (string)c_str;
}


// *************************************************************************
// Draw 2d-plot
void 
draw_plot( string datfile, string Rfile, string epsfile, 
	   string xlab, string ylab ){
  ofstream ofs( Rfile.c_str() );
  string R = (string)"\n"
    + "postscript(\"" + epsfile + "\")"                           + "\n"
    + "data <- read.table(\"" + datfile + "\")"                   + "\n"
    + "plot(data, "
    + "main=\"" + datfile + "\", "
    + "xlab=\""+ xlab + "\", ylab=\"" + ylab + "\")"              + "\n"
    + "dev.off()"                                                 + "\n";
  ofs << R;
  ofs.close();
  string cmd = (string)"R CMD BATCH " + Rfile;
  system(cmd.c_str());
  cmd = (string)"rm *Rout " + datfile + " " + Rfile;
  system(cmd.c_str());
}


// *************************************************************************
// Draw Heat map
void draw_heatmap( string datfile, string Rfile, string pdffile, 
		   unsigned num_cells, string Rdir, bool flg_rev ){
  int stt = -1 * (int)num_cells;
  int end = (int)num_cells;
  ofstream ofs( Rfile.c_str()  );
  string col;
  if( flg_rev ){
    col = (string)"rev( heat.colors(16) )";
  } else {
    col = (string)"heat.colors(16)";
  }
  string R = (string)"\n"
    + "library(gplots)"                                            + "\n"
    + "source(\"" + Rdir + "my_heatmap.r\")"                       + "\n"
    + "pdf(\"" + pdffile + "\")"                                   + "\n"
    + "data <- read.table(\"" + datfile + "\")"                    + "\n"
    + "my_heatmap( as.matrix(data), "                              
    + "col = " + col + ", "                             
    + "labCol = " + int2string(stt) + ":" + int2string(end) + ", " 
    + "labRow = " + int2string(stt) + ":" + int2string(end) + ", " 
    + ")"                                                          + "\n"
    + "dev.off()"                                                  + "\n";
  ofs << R;
  ofs.close();
  string cmd = (string)"R CMD BATCH " + Rfile;
  system(cmd.c_str());
  cmd = (string)"rm *Rout " + datfile + " " + Rfile;
  system(cmd.c_str());
}


// *************************************************************************
// Draw Heat map
void draw_heatmap( string datfile, string Rfile, string pdffile, 
		   unsigned n_cols, unsigned n_rows, 
		   string Rdir, bool flg_rev ){
  string labCol = int2string(1) + ":" + int2string(n_cols);
  string labRow = int2string(n_rows) + ":" + int2string(1);
  ofstream ofs( Rfile.c_str()  );
  string col;
  if( flg_rev ){
    col = (string)"rev( heat.colors(16) )";
  } else {
    col = (string)"heat.colors(16)";
  }
  string R = (string)"\n"
    + "library(gplots)"                                            + "\n"
    + "source(\"" + Rdir + "my_heatmap.r\")"                       + "\n"
    + "pdf(\"" + pdffile + "\")"                                   + "\n"
    + "data <- read.table(\"" + datfile + "\")"                    + "\n"
    + "my_heatmap( as.matrix(data), "                              
    + "col = " + col + ", "                             
    + "labCol = " + labCol + ", " 
    + "labRow = " + labRow + ", " 
    + ")"                                                          + "\n"
    + "dev.off()"                                                  + "\n";
  ofs << R;
  ofs.close();
  string cmd = (string)"R CMD BATCH " + Rfile;
  system(cmd.c_str());
  cmd = (string)"rm *Rout " + datfile + " " + Rfile;
  system(cmd.c_str());
}
