// *************************************************************************
//  weight_matrix.cpp
// *************************************************************************

#include <ctime>
#include "weight_matrix.h"
#include "wm_options.h"
#include "position_matrix.h"
#include "wm_matrix.h"

// *************************************************************************
// main function.
int main( int argc, char* argv[] ){
  // Get options
  cerr << endl;
  cerr << "Getting options...";
  wm_options opt( argc, argv );
  opt.print();

  // Record start time
  time_t sttTime = time(0);

  // Compute diagonal pseudo distance (DPD)
  cerr << "Computing diagonal pseudo distance..." << endl;
  time_t baseTime = time(0);
  unsigned num_cells = opt.get_num_cells();
  double   col_index = opt.get_col_index();
  double   sds       = opt.get_sds();
  position_matrix p_mat( num_cells );
  wm_matrix wm_mat( p_mat, num_cells, col_index, sds );
  cerr << p_mat.size() << " cells were created." << endl;
  cerr << "done ("
       << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Display DPD matrix
  cerr << "Displaying DPD matrix..." << endl;
  wm_mat.d_display();
  cerr << endl << endl;

  // Display weight matrix
  cerr << "Displaying weight matrix..." << endl;
  wm_mat.w_display();
  cerr << endl << endl;

  // Draw heat maps
  cerr << "Drawing heat maps..." << endl;
  baseTime = time(0);
  string jobname = opt.get_jobname();
  string Rdir    = opt.get_Rdir();
  wm_mat.draw( jobname, Rdir );
  cerr << "done ("
       << time(0) - baseTime << " seconds)" << endl;
  cerr << endl;

  // Report total time required for this program
  cerr << "Total time : " << time(0) - sttTime << " seconds" << endl;

  return 0;
}
