#ifndef __INC__WM_MATRIX_H__
#define __INC__WM_MATRIX_H__

#include <cmath>
#include <map>
#include "position_matrix.h"

using namespace std;

class wm_matrix {
  unsigned num_cells;
  mat_idx origin;
  map< mat_idx, unsigned > idx2dpd;
  map< mat_idx, double > idx2weight;

  void insert( mat_idx idx, unsigned dpd, double weight );
 public:
  wm_matrix( position_matrix& p_mat, unsigned n_cells, 
	     double col_index, double sds );
  ~wm_matrix() { idx2dpd.clear(); idx2weight.clear(); }

  map< mat_idx, unsigned >::iterator d_begin() { return idx2dpd.begin(); }
  map< mat_idx, unsigned >::iterator d_end() { return idx2dpd.end(); }
  map< mat_idx, unsigned >::iterator d_find( mat_idx mi )
  { return idx2dpd.find( mi ); }
  map< mat_idx, double >::iterator w_begin() { return idx2weight.begin(); }
  map< mat_idx, double >::iterator w_end() { return idx2weight.end(); }
  map< mat_idx, double >::iterator w_find( mat_idx mi )
  { return idx2weight.find( mi ); }

  unsigned get_dpd( unsigned i, unsigned j );
  unsigned get_dpd( mat_idx mi );
  double get_weight( unsigned i, unsigned j );
  double get_weight( mat_idx mi );
  double dpd2weight( unsigned dpd, double sds );

  void d_display();
  void d_display( unsigned n_cells );
  void d_print( string file_name );
  void w_display();
  void w_display( unsigned n_cells );
  void w_print( string file_name );

  void draw( string jobname, string Rdir );
  void draw_dpd( string name, string Rdir );
  void draw_weight( string name, string Rdir );
};

#endif // __INC__WM_MATRIX_H__
