// *************************************************************************
//  wm_options.cpp
// *************************************************************************

#include "wm_options.h"
using namespace std;

// *************************************************************************
// Class wm_options
// Constructor
// Parse command line and get options
wm_options::wm_options( int argc, char* argv[] ){
  char opt;
  int opt_idx;
  bool flg_N = false;
  bool flg_L = false;
  bool flg_S = false;
  bool flg_O = false;
  bool flg_R = false;
  bool flg_H = false;

  string usage;
  usage = (string)"\n"
    + "********************************************************"  + "\n"
    + "Description:"                                              + "\n"
    + "  weight_matrix"                                           + "\n"
    + "    --  Draw 2d matrix that visualizes the relation "      + "\n"
    + "       between DPD value and weight"                       + "\n"
                                                                  + "\n"
    + "Options:"                                                  + "\n"
    + "  -n  set the number of cells"                             + "\n"
    + "  -l  set collinearity index"                              + "\n"
    + "  -s  set standard deviation squared value for Gaussian"   + "\n"
    + "  -r  set directory containing R scripts"                  + "\n"
    + "  -o  set output directory"                                + "\n"
    + "  -h  print this message"                                  + "\n"
    + "********************************************************"  + "\n";

  opt_idx = 0;
  while( -1 ){
    opt = getopt( argc, argv, "n:l:s:r:o:h" );
    opt_idx++;
    if( opt == -1 ){
      break;
    }
    switch( opt ){
    case 'n':
      num_cells = atoi( argv[++opt_idx] );
      flg_N = true;
      break;
    case 'l':
      col_index = atof( argv[++opt_idx] );
      flg_L = true;
      break;
    case 's':
      sds = atof( argv[++opt_idx] );
      flg_S = true;
      break;
    case 'r':
      Rdir = (string)argv[++opt_idx];
      flg_R = true;
      break;
    case 'o':
      out_dir = (string)argv[++opt_idx];
      flg_O = true;
      break;
    case 'h':
      flg_H = true;
      break;
    case '?':
    default:
      cerr << "Unknown option " << opt << endl;
      exit( -1 );
    }
  }
  if( flg_H || !flg_N || !flg_L || !flg_S || !flg_R || !flg_O ){
    cerr << usage << endl;
    exit(1);
  }
  set_file_name();
}


// *************************************************************************
// Class wm_options
// Set file names
void wm_options::set_file_name() {
  jobname 
    = out_dir + "/" + "n_" + int2string( (int)num_cells ) 
    + "." + "col_" + double2string( col_index )
    + "." + "sds_" + double2string( sds );
}


// *************************************************************************
// Class wm_options
// Print options
void wm_options::print() {
  string options = (string)"\n"
    + "********************************************************"  + "\n"
    + "Options:"                                                  + "\n"
    + "Number of cells     : " + int2string( (int)num_cells )     + "\n"
    + "Collinearity index  : " + double2string( col_index )       + "\n"
    + "SDS for Gaussian    : " + double2string( sds )             + "\n"
    + "R directory         : " + Rdir                             + "\n"
    + "Output directory    : " + out_dir                          + "\n"
    + "********************************************************"  + "\n";
  cerr << options << endl;
}
